@@ -6,18 +6,20 @@
    This file is part of GNUstep.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef CairoGState_h
@@ -34,12 +36,14 @@
   @public
     cairo_t *_ct;
     CairoSurface *_surface;
+
+    int _strokeadjust;
 }
 
-- (void) GSCurrentDevice: (void **)device: (int *)x : (int *)y;
-- (void) GSSetDevice: (void *)device : (int)x : (int)y;
+- (void) GSCurrentSurface: (CairoSurface **)surface: (int *)x : (int *)y;
+- (void) GSSetSurface: (CairoSurface *)surface : (int)x : (int)y;
 
+- (void) showPage;
 @end
 
 #endif
-
