@@ -67,6 +67,11 @@
 #define GET_Y_LPARAM(p) ((int)(short)HIWORD(p))
 #endif
 
+// The horizontal mousehweel message is missing in MinGW 
+#ifndef WM_MOUSEHWHEEL
+#define WM_MOUSEHWHEEL 0x020E
+#endif
+
 #define EVENT_WINDOW(lp) (GSWindowWithNumber((int)lp)) 
 
 DWORD windowStyleForGSStyle(unsigned int style);
@@ -111,6 +116,8 @@ typedef struct w32serverFlags {
 
 @interface WIN32Server (w32_activate)
 
+- (LRESULT) decodeWM_MOUSEACTIVATEParams: (WPARAM)wParam : (LPARAM)lParam 
+						: (HWND)hwnd;
 - (LRESULT) decodeWM_ACTIVEParams: (WPARAM)wParam : (LPARAM)lParam 
 				 : (HWND)hwnd;
 - (LRESULT) decodeWM_ACTIVEAPPParams: (HWND)hwnd : (WPARAM)wParam 
@@ -130,7 +137,7 @@ typedef struct w32serverFlags {
 - (LRESULT) decodeWM_GETMINMAXINFOParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd;
 - (LRESULT) decodeWM_EXITSIZEMOVEParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd;
 - (LRESULT) decodeWM_MOVINGParams: (HWND)hwnd : (WPARAM)wParam : (LPARAM)lParam;
-- (void) decodeWM_SIZINGParams: (HWND)hwnd : (WPARAM)wParam : (LPARAM)lParam;
+- (LRESULT) decodeWM_SIZINGParams: (HWND)hwnd : (WPARAM)wParam : (LPARAM)lParam;
 
 @end
 
