@@ -52,7 +52,8 @@ draw_info_t ART_DI;
   unsigned char r,g,b;
 
   [super setColor: color  state: cState];
-  if (cState&(COLOR_FILL|COLOR_STROKE))
+
+  if (cState & COLOR_FILL)
     {
       c = fillColor;
       gsColorToRGB(&c); /* TODO: check this */
@@ -66,20 +67,30 @@ draw_info_t ART_DI;
       if (c.field[2] < 0.0) c.field[2] = 0.0;
       b = c.field[2] * 255;
 
-      if (cState & COLOR_FILL)
-        {
-	  fill_color[0] = r;
-	  fill_color[1] = g;
-	  fill_color[2] = b;
-	  fill_color[3] = fillColor.field[AINDEX] * 255;
-	}
-      if (cState & COLOR_STROKE)
-        {
-	  stroke_color[0] = r;
-	  stroke_color[1] = g;
-	  stroke_color[2] = b;
-	  stroke_color[3] = strokeColor.field[AINDEX] * 255;
-	}
+      fill_color[0] = r;
+      fill_color[1] = g;
+      fill_color[2] = b;
+      fill_color[3] = fillColor.field[AINDEX] * 255;
+    }
+
+  if (cState & COLOR_STROKE)
+    {
+      c = strokeColor;
+      gsColorToRGB(&c); /* TODO: check this */
+      if (c.field[0] > 1.0) c.field[0] = 1.0;
+      if (c.field[0] < 0.0) c.field[0] = 0.0;
+      r = c.field[0] * 255;
+      if (c.field[1] > 1.0) c.field[1] = 1.0;
+      if (c.field[1] < 0.0) c.field[1] = 0.0;
+      g = c.field[1] * 255;
+      if (c.field[2] > 1.0) c.field[2] = 1.0;
+      if (c.field[2] < 0.0) c.field[2] = 0.0;
+      b = c.field[2] * 255;
+
+      stroke_color[0] = r;
+      stroke_color[1] = g;
+      stroke_color[2] = b;
+      stroke_color[3] = strokeColor.field[AINDEX] * 255;
     }
 }
 
@@ -627,6 +638,7 @@ draw_info_t ART_DI;
   di.bytes_per_pixel = DI.bytes_per_pixel;
   di.inline_alpha = DI.inline_alpha;
   di.inline_alpha_ofs = DI.inline_alpha_ofs;
+  di.byte_order = ImageByteOrder(window->display);
 
   ASSIGN(wi, [XWindowBuffer windowBufferForWindow: window depthInfo: &di]);
 }
@@ -650,3 +662,72 @@ draw_info_t ART_DI;
 }
 
 @end
+
+@implementation ARTGState (PatternColor)
+typedef struct _SavedClip {
+  int clip_x0,clip_y0,clip_x1,clip_y1;
+  BOOL all_clipped;
+  int clip_sx,clip_sy;
+  unsigned int *clip_span;
+  unsigned int *clip_index;
+  int clip_num_span;
+} SavedClip;
+
+- (void *) saveClip
+{
+  SavedClip *savedClip = malloc(sizeof(SavedClip));
+  int i;
+
+  savedClip->clip_x0 = clip_x0;
+  savedClip->clip_y0 = clip_y0;
+  savedClip->clip_x1 = clip_x1;
+  savedClip->clip_y1 = clip_y1;
+  savedClip->all_clipped = all_clipped;
+  savedClip->clip_sx = clip_sx;
+  savedClip->clip_sy = clip_sy;
+  if (clip_num_span == 0)
+    {
+      savedClip->clip_span = NULL;
+      savedClip->clip_index = NULL;
+    }
+  else
+    {
+      savedClip->clip_span = malloc(sizeof(int) * clip_num_span);
+      savedClip->clip_index = malloc(sizeof(int) * clip_num_span);
+      for (i = 0; i < clip_num_span; i++)
+        {
+          savedClip->clip_span[i] = clip_span[i];
+        }
+      for (i = 0; i < clip_num_span; i++)
+        {
+          savedClip->clip_index[i] = clip_index[i];
+        }
+    }
+  savedClip->clip_num_span = clip_num_span;
+
+  return savedClip;
+}
+
+- (void) restoreClip: (void *)saved
+{
+  SavedClip *savedClip = (SavedClip *)saved;
+
+  clip_x0 = savedClip->clip_x0;
+  clip_y0 = savedClip->clip_y0;
+  clip_x1 = savedClip->clip_x1;
+  clip_y1 = savedClip->clip_y1;
+  all_clipped = savedClip->all_clipped;
+  clip_sx = savedClip->clip_sx;
+  clip_sy = savedClip->clip_sy;
+  if (clip_span)
+    {
+      free(clip_span);
+      free(clip_index);
+    }
+  clip_span = savedClip->clip_span;
+  clip_index = savedClip->clip_index;
+  clip_num_span = savedClip->clip_num_span;
+  free(savedClip);
+}
+
+@end
