@@ -207,6 +207,8 @@ static NSArray *faFromFc(FcPattern *pat)
     return @"Bitstream Vera Sans";
   if ([allFontNames containsObject: @"FreeSans"])
     return @"FreeSans";
+  if ([allFontNames containsObject: @"DejaVu Sans"])
+    return @"DejaVu Sans";
   return @"Helvetica";
 }
 
@@ -216,6 +218,8 @@ static NSArray *faFromFc(FcPattern *pat)
     return @"Bitstream Vera Sans-Bold";
   if ([allFontNames containsObject: @"FreeSans-Bold"])
     return @"FreeSans-Bold";
+  if ([allFontNames containsObject: @"DejaVu Sans-Bold"])
+    return @"DejaVu Sans-Bold";
   return @"Helvetica-Bold";
 }
 
@@ -225,6 +229,8 @@ static NSArray *faFromFc(FcPattern *pat)
     return @"Bitstream Vera Sans Mono";
   if ([allFontNames containsObject: @"FreeMono"])
     return @"FreeMono";
+  if ([allFontNames containsObject: @"DejaVu Sans Mono"])
+    return @"DejaVu Sans Mono";
   return @"Courier";
 }
 
