@@ -30,6 +30,7 @@
 #include <AppKit/NSAffineTransform.h>
 #include <AppKit/NSBezierPath.h>
 #include <AppKit/NSColor.h>
+#include <AppKit/NSGradient.h>
 #include <AppKit/NSGraphics.h>
 #include "cairo/CairoGState.h"
 #include "cairo/CairoFontInfo.h"
@@ -89,7 +90,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   NSAffineTransform *ictm;
   
-  ictm = [ctm copyWithZone: GSObjCZone(ctm)];
+  ictm = [ctm copyWithZone: [ctm zone]];
   [ictm invert];
   f = floatFromUserSpace(ictm, f);
   RELEASE(ictm);
@@ -296,22 +297,6 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 /*
  * Color operations
  */
-- (void) setColor: (device_color_t *)color state: (color_state_t)cState
-{
-  device_color_t c;
-
-  [super setColor: color state: cState];
-  if (_ct == NULL)
-    {
-      // Window device isn't set yet
-      return;
-    }
-  c = fillColor;
-  gsColorToRGB(&c);
-  // The underlying concept does not allow to determine if alpha is set or not.
-  cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
-}
-
 - (void) GSSetPatterColor: (NSImage*)image 
 {
   // FIXME: Create a cairo surface from the image and set it as source.
@@ -382,6 +367,12 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
       cairo_matrix_t saved_matrix;
       cairo_matrix_t local_matrix;
       NSPoint        p = [path currentPoint];
+      device_color_t c;
+
+      c = strokeColor;
+      gsColorToRGB(&c);
+      // The underlying concept does not allow to determine if alpha is set or not.
+      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
 
       cairo_get_matrix(_ct, &saved_matrix);
 
@@ -399,7 +390,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 - (void) GSSetFont: (GSFontInfo *)fontref
 {
   cairo_matrix_t font_matrix;
-  const float *matrix; 
+  const CGFloat *matrix; 
 
   [super GSSetFont: fontref];
 
@@ -426,12 +417,18 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      GS_BEGINITEMBUF(c, length + 1, char);
+      GS_BEGINITEMBUF(chars, length + 1, char);
+      device_color_t c;
+
+      c = strokeColor;
+      gsColorToRGB(&c);
+      // The underlying concept does not allow to determine if alpha is set or not.
+      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
 
       [self _setPoint];
-      memcpy(c, string, length);
-      c[length] = 0;
-      cairo_show_text(_ct, c);
+      memcpy(chars, string, length);
+      chars[length] = 0;
+      cairo_show_text(_ct, chars);
       GS_ENDITEMBUF();
     }
 }
@@ -442,6 +439,12 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
     {
       cairo_matrix_t local_matrix;
       NSAffineTransformStruct	matrix = [ctm transformStruct];
+      device_color_t c;
+
+      c = strokeColor;
+      gsColorToRGB(&c);
+      // The underlying concept does not allow to determine if alpha is set or not.
+      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
 
       [self _setPoint];
       // FIXME: Hack to get font in rotated view working
@@ -497,9 +500,6 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
   cairo_matrix_translate(&local_matrix, 0,  -[_surface size].height);
   cairo_set_matrix(_ct, &local_matrix);
 
-  // super call did go to the old _ct, so redo it
-  [self setColor: &fillColor state: COLOR_BOTH];
-
   // Cairo's default line width is 2.0
   cairo_set_line_width(_ct, 1.0);
   cairo_set_operator(_ct, CAIRO_OPERATOR_OVER);
@@ -512,8 +512,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
-      *flatness = (float)cairo_get_tolerance(_ct);
-      *flatness = floatToUserSpace(ctm, *flatness);
+      *flatness = (float)cairo_get_tolerance(_ct) * 2;
     }
 }
 
@@ -621,7 +620,9 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
   [super DPSsetflat: flatness];
   if (_ct)
     {
-      cairo_set_tolerance(_ct, floatFromUserSpace(ctm, flatness));
+      // Divide GNUstep flatness by 2 to get Cairo tolerance - this produces
+      // results visually similar to OS X.
+      cairo_set_tolerance(_ct, flatness / 2);
     }
 }
 
@@ -747,7 +748,7 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
     {
       float offs;
 
-      if ((remainderf(cairo_get_line_width(_ct), 2.0) == 0.0)
+      if ((remainderf((float)cairo_get_line_width(_ct), 2.0) == 0.0)
           || fillOrClip == YES)
         offs = 0.0;
       else
@@ -810,6 +811,18 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
+      device_color_t c;
+
+      if (pattern != nil)
+        {
+          [self eofillPath: path withPattern: pattern];
+          return;
+        }
+
+      c = fillColor;
+      gsColorToRGB(&c);
+      // The underlying concept does not allow to determine if alpha is set or not.
+      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
       [self _setPath:YES];
       cairo_set_fill_rule(_ct, CAIRO_FILL_RULE_EVEN_ODD);
       cairo_fill(_ct);
@@ -822,6 +835,18 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
+      device_color_t c;
+
+      if (pattern != nil)
+        {
+          [self fillPath: path withPattern: pattern];
+          return;
+        }
+
+      c = fillColor;
+      gsColorToRGB(&c);
+      // The underlying concept does not allow to determine if alpha is set or not.
+      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
       [self _setPath:YES];
       cairo_fill(_ct);
     }
@@ -840,6 +865,12 @@ static float floatToUserSpace(NSAffineTransform *ctm, float f)
 {
   if (_ct)
     {
+      device_color_t c;
+
+      c = strokeColor;
+      gsColorToRGB(&c);
+      // The underlying concept does not allow to determine if alpha is set or not.
+      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
       [self _setPath:NO];
       cairo_stroke(_ct);
     }
@@ -1176,6 +1207,10 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
       cairo_matrix_init_scale(&local_matrix, 1, -1);
       cairo_matrix_translate(&local_matrix, 0, -2*pixelsHigh);
       cairo_pattern_set_matrix(cpattern, &local_matrix);
+      if (cairo_version() >= CAIRO_VERSION_ENCODE(1, 6, 0))
+        {
+          cairo_pattern_set_extend(cpattern, CAIRO_EXTEND_PAD);
+        }
       cairo_set_source(_ct, cpattern);
       cairo_pattern_destroy(cpattern);
 
@@ -1190,6 +1225,10 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
       cairo_matrix_init_scale(&local_matrix, 1, -1);
       cairo_matrix_translate(&local_matrix, 0, -pixelsHigh);
       cairo_pattern_set_matrix(cpattern, &local_matrix);
+      if (cairo_version() >= CAIRO_VERSION_ENCODE(1, 6, 0))
+        {
+          cairo_pattern_set_extend(cpattern, CAIRO_EXTEND_PAD);
+        }
       cairo_set_source(_ct, cpattern);
       cairo_pattern_destroy(cpattern);
 
@@ -1211,10 +1250,15 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
   if (_ct)
     {
       NSBezierPath *oldPath = path;
+      device_color_t c;
 
       cairo_save(_ct);
       _set_op(_ct, op);
 
+      c = fillColor;
+      gsColorToRGB(&c);
+      // The underlying concept does not allow to determine if alpha is set or not.
+      cairo_set_source_rgba(_ct, c.field[0], c.field[1], c.field[2], c.field[AINDEX]);
       // This is almost a rectclip::::, but the path stays unchanged.
       path = [NSBezierPath bezierPathWithRect: aRect];
       [path transformUsingAffineTransform: ctm];
@@ -1287,6 +1331,13 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
   }
 
   [source->ctm boundingRectFor: aRect result: &aRect];
+  if (cairo_version() >= CAIRO_VERSION_ENCODE(1, 8, 0))
+    {
+      NSSize size = [source->_surface size];
+      
+      // For cairo > 1.8 we seem to need this adjustment
+      aRect.origin.y -= 2*(source->offset.y - size.height);
+    }
 
   x = floorf(aPoint.x);
   y = floorf(aPoint.y + 0.5);
@@ -1331,28 +1382,148 @@ _set_op(cairo_t *ct, NSCompositingOperation op)
   cairo_restore(_ct);
 }
 
-- (void) compositeGState: (CairoGState *)source 
-		fromRect: (NSRect)aRect 
-		 toPoint: (NSPoint)aPoint 
-		      op: (NSCompositingOperation)op
+@end
+
+@implementation CairoGState (PatternColor)
+
+- (void *) saveClip
 {
-  [self compositeGState: source 
-	       fromRect: aRect 
-		toPoint: aPoint 
-		     op: op
-	       fraction: 1.0];
+#if CAIRO_VERSION > CAIRO_VERSION_ENCODE(1, 4, 0)
+  cairo_status_t status;
+  cairo_rectangle_list_t *clip_rects = cairo_copy_clip_rectangle_list(_ct);
+
+  status = cairo_status(_ct);
+  if (status == CAIRO_STATUS_SUCCESS)
+    {
+      return clip_rects;
+    }
+#endif
+
+  return NULL;
 }
 
-- (void) dissolveGState: (CairoGState *)source
-	       fromRect: (NSRect)aRect
-		toPoint: (NSPoint)aPoint 
-		  delta: (float)delta
+- (void) restoreClip: (void *)savedClip
 {
-  [self compositeGState: source 
-	       fromRect: aRect 
-		toPoint: aPoint 
-		     op: NSCompositeSourceOver
-	       fraction: delta];
+#if CAIRO_VERSION > CAIRO_VERSION_ENCODE(1, 4, 0)
+  if (savedClip)
+    {
+      int i;
+      cairo_rectangle_list_t *clip_rects = (cairo_rectangle_list_t *)savedClip;
+      
+      cairo_reset_clip(_ct);
+      if (cairo_version() >= CAIRO_VERSION_ENCODE(1, 6, 0))
+        {
+          for (i = 0; i < clip_rects->num_rectangles; i++)
+            {
+              cairo_rectangle_t rect = clip_rects->rectangles[i];
+              
+              cairo_rectangle(_ct, rect.x, rect.y, 
+                              rect.width, rect.height);
+              cairo_clip(_ct);
+            }
+        }
+      else
+        {
+          for (i = 0; i < clip_rects->num_rectangles; i++)
+            {
+              cairo_rectangle_t rect = clip_rects->rectangles[i];
+              NSSize size = [_surface size];
+              
+              cairo_rectangle(_ct, rect.x, 
+                              /* This strange computation is due 
+                                 to the device offset missing for 
+                                 clip rects in cairo < 1.6.0.  */
+                              rect.y + 2*(offset.y - size.height), 
+                              rect.width, rect.height);
+              cairo_clip(_ct);
+            }
+        }
+      cairo_rectangle_list_destroy(clip_rects);
+    }
+#endif
+}
+
+@end
+
+@implementation CairoGState (NSGradient)
+
+- (void) drawGradient: (NSGradient*)gradient
+           fromCenter: (NSPoint)startCenter
+               radius: (CGFloat)startRadius
+             toCenter: (NSPoint)endCenter 
+               radius: (CGFloat)endRadius
+              options: (NSUInteger)options
+{
+  int i;
+  int stops = [gradient numberOfColorStops];
+  NSPoint startP = [ctm transformPoint: startCenter];
+  NSPoint endP = [ctm transformPoint: endCenter];
+  cairo_pattern_t *cpattern = cairo_pattern_create_radial(startP.x, startP.y, 
+                                                          floatFromUserSpace(ctm, startRadius),
+                                                          endP.x, endP.y, 
+                                                          floatFromUserSpace(ctm, endRadius));
+  for (i = 0; i < stops; i++)
+    {
+      NSColor *color;
+      CGFloat location;
+      double red;
+      double green;
+      double blue;
+      double alpha;
+
+      [gradient getColor: &color
+                location: &location
+                atIndex: i];
+      red = [color redComponent];
+      green = [color greenComponent];
+      blue = [color blueComponent];
+      alpha = [color alphaComponent];
+      cairo_pattern_add_color_stop_rgba(cpattern, location,
+                                        red, green, blue, alpha);
+    }
+  cairo_save(_ct);
+  cairo_set_source(_ct, cpattern);
+  cairo_pattern_destroy(cpattern);
+  cairo_paint(_ct);
+  cairo_restore(_ct);
+}
+
+- (void) drawGradient: (NSGradient*)gradient
+            fromPoint: (NSPoint)startPoint
+              toPoint: (NSPoint)endPoint
+              options: (NSUInteger)options
+{
+  int i;
+  int stops = [gradient numberOfColorStops];
+  NSPoint startP = [ctm transformPoint: startPoint];
+  NSPoint endP = [ctm transformPoint: endPoint];
+  cairo_pattern_t *cpattern = cairo_pattern_create_linear(startP.x, startP.y,
+                                                          endP.x, endP.y);
+
+  for (i = 0; i < stops; i++)
+    {
+      NSColor *color;
+      CGFloat location;
+      double red;
+      double green;
+      double blue;
+      double alpha;
+
+      [gradient getColor: &color
+                location: &location
+                atIndex: i];
+      red = [color redComponent];
+      green = [color greenComponent];
+      blue = [color blueComponent];
+      alpha = [color alphaComponent];
+      cairo_pattern_add_color_stop_rgba(cpattern, location,
+                                        red, green, blue, alpha);
+    }
+  cairo_save(_ct);
+  cairo_set_source(_ct, cpattern);
+  cairo_pattern_destroy(cpattern);
+  cairo_paint(_ct);
+  cairo_restore(_ct);
 }
 
 @end
