@@ -778,20 +778,11 @@ static NSMapTable *gtable;
 - (void) DPScomposite: (float)x : (float)y : (float)w : (float)h 
 		     : (int)gstateNum : (float)dx : (float)dy : (int)op
 {
-  NSRect rect;
-  NSPoint p;
-  GSGState *g = gstate;
-
-  if (gstateNum)
-    {
-      [self DPSexecuserobject: gstateNum];
-      ctxt_pop(g, opstack, GSGState);
-    }
-
-  rect = NSMakeRect(x, y, w, h);
-  p    = NSMakePoint(dx, dy);
-
-  [gstate compositeGState: g fromRect: rect toPoint: p op: op];
+  [self GScomposite: gstateNum
+        toPoint: NSMakePoint(dx, dy)
+        fromRect: NSMakeRect(x, y, w, h)
+        operation: op
+        fraction: 1.0];
 }
 
 - (void) DPScompositerect: (float)x : (float)y : (float)w : (float)h : (int)op
@@ -802,20 +793,11 @@ static NSMapTable *gtable;
 - (void) DPSdissolve: (float)x : (float)y : (float)w : (float)h 
 		    : (int)gstateNum : (float)dx : (float)dy : (float)delta
 {
-  NSRect rect;
-  NSPoint p;
-  GSGState *g = gstate;
-
-  if (gstateNum)
-    {
-      [self DPSexecuserobject: gstateNum];
-      ctxt_pop(g, opstack, GSGState);
-    }
-
-  rect = NSMakeRect(x, y, w, h);
-  p    = NSMakePoint(dx, dy);
-
-  [gstate dissolveGState: g fromRect: rect toPoint: p delta: delta];
+  [self GScomposite: gstateNum
+        toPoint: NSMakePoint(dx, dy)
+        fromRect: NSMakeRect(x, y, w, h)
+        operation: NSCompositeSourceOver
+        fraction: delta];
 }
 
 - (void) GScomposite: (int)gstateNum
@@ -842,15 +824,15 @@ static NSMapTable *gtable;
 - (void) GSDrawImage: (NSRect) rect: (void *) imageref
 {
   NSBitmapImageRep *bitmap;
-  const unsigned char *data[5];
+  unsigned char *data[5];
 
   bitmap = (NSBitmapImageRep*)imageref;
-  [bitmap getBitmapDataPlanes: &data];
+  [bitmap getBitmapDataPlanes: data];
   [self NSDrawBitmap: rect : [bitmap pixelsWide] : [bitmap pixelsHigh]
         : [bitmap bitsPerSample] : [bitmap samplesPerPixel]
         : [bitmap bitsPerPixel] : [bitmap bytesPerRow] : [bitmap isPlanar]
         : [bitmap hasAlpha] :  [bitmap colorSpaceName]
-        : data];
+        : (const unsigned char**)data];
 }
 
 /* ----------------------------------------------------------------------- */
@@ -1019,3 +1001,32 @@ static NSMapTable *gtable;
   ctxt_pop(obj, opstack, NSObject);
 }
 @end
+
+@implementation GSContext (NSGradient)
+- (void) drawGradient: (NSGradient*)gradient
+           fromCenter: (NSPoint)startCenter
+               radius: (CGFloat)startRadius
+             toCenter: (NSPoint)endCenter 
+               radius: (CGFloat)endRadius
+              options: (NSUInteger)options
+{
+  [gstate drawGradient: gradient
+          fromCenter: startCenter
+          radius: startRadius
+          toCenter: endCenter 
+          radius: endRadius
+          options: options];
+}
+
+- (void) drawGradient: (NSGradient*)gradient
+            fromPoint: (NSPoint)startPoint
+              toPoint: (NSPoint)endPoint
+              options: (NSUInteger)options
+{
+  [gstate drawGradient: gradient
+          fromPoint: startPoint
+          toPoint: endPoint
+          options: options];
+}
+
+@end
