@@ -1,6 +1,6 @@
 /* GSGState - Generic graphic state
 
-   Copyright (C) 1998 Free Software Foundation, Inc.
+   Copyright (C) 1998-2010 Free Software Foundation, Inc.
 
    Written by:  Adam Fedor <fedor@gnu.org>
    Date: Mar 2002
@@ -25,19 +25,20 @@
 */
 
 #include "config.h"
-#include <Foundation/NSObjCRuntime.h>
-#include <AppKit/NSAffineTransform.h>
-#include <AppKit/NSBezierPath.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/NSColorSpace.h>
-#include <AppKit/NSImage.h>
-#include <GNUstepGUI/GSFontInfo.h>
-#include <AppKit/NSGraphics.h>
-#include "gsc/GSContext.h"
-#include "gsc/GSGState.h"
-#include "gsc/GSFunction.h"
+#import <Foundation/NSObjCRuntime.h>
+#import <Foundation/NSValue.h>
+#import <AppKit/NSAffineTransform.h>
+#import <AppKit/NSBezierPath.h>
+#import <AppKit/NSColor.h>
+#import <AppKit/NSColorSpace.h>
+#import <AppKit/NSImage.h>
+#import <GNUstepGUI/GSFontInfo.h>
+#import <AppKit/NSGraphics.h>
+#import "gsc/GSContext.h"
+#import "gsc/GSGState.h"
+#import "gsc/GSFunction.h"
 #include "math.h"
-#include <GNUstepBase/Unicode.h>
+#import <GNUstepBase/Unicode.h>
 
 #define CHECK_PATH \
   if (!path) \
@@ -78,7 +79,7 @@
 
 - (id) deepen
 {
-  NSZone *zone = GSObjCZone(self);
+  NSZone *zone = [self zone];
 
   if (path)
     self->path = [path copyWithZone: zone];
@@ -120,15 +121,10 @@
     in the current color */
 - (void) setColor: (device_color_t *)color state: (color_state_t)cState
 {
-  float alpha;
-  alpha = fillColor.field[AINDEX];
   if (cState & COLOR_FILL)
     fillColor = *color;
-  fillColor.field[AINDEX] = alpha;
-  alpha = strokeColor.field[AINDEX];
   if (cState & COLOR_STROKE)
     strokeColor = *color;
-  strokeColor.field[AINDEX] = alpha;
   cstate = cState;
   DESTROY(pattern);
 }
@@ -138,20 +134,31 @@
   ASSIGN(pattern, image);
 }
 
+// This is only a fall back, the method should not be called any more.
 - (void) compositeGState: (GSGState *)source
                 fromRect: (NSRect)aRect
                  toPoint: (NSPoint)aPoint
                       op: (NSCompositingOperation)op
 {
-  [self subclassResponsibility: _cmd];
+  [self compositeGState: source 
+        fromRect: aRect 
+        toPoint: aPoint 
+        op: op
+        fraction: 1.0];
+
 }
 
+// This is only a fall back, the method should not be called any more.
 - (void) dissolveGState: (GSGState *)source
                fromRect: (NSRect)aRect
                 toPoint: (NSPoint)aPoint
                   delta: (float)delta
 {
-  [self subclassResponsibility: _cmd];
+  [self compositeGState: source 
+        fromRect: aRect 
+        toPoint: aPoint 
+        op: NSCompositeSourceOver
+        fraction: delta];
 }
 
 - (void) compositeGState: (GSGState *)source
@@ -160,21 +167,7 @@
                       op: (NSCompositingOperation)op
                 fraction: (float)delta
 {
-  if (op == NSCompositeSourceOver)
-    {
-      [self dissolveGState: source
-	          fromRect: aRect
-                   toPoint: aPoint
-	             delta: delta];
-    }
-  else
-    {
-      [self compositeGState: source
-	          fromRect: aRect
-                   toPoint: aPoint
-	                op: op];
-    }
-
+  [self subclassResponsibility: _cmd];
 }
 
 - (void) compositerect: (NSRect)aRect
@@ -261,6 +254,8 @@
   CLAMP(y)
   CLAMP(k)
   gsMakeColor(&col, cmyk_colorspace, c, m, y, k);
+  // Keep the old alpha value
+  col.field[AINDEX] = fillColor.field[AINDEX];
   [self setColor: &col state: COLOR_BOTH];
 }
 
@@ -269,6 +264,8 @@
   device_color_t col;
   CLAMP(gray)
   gsMakeColor(&col, gray_colorspace, gray, 0, 0, 0);
+  // Keep the old alpha value
+  col.field[AINDEX] = fillColor.field[AINDEX];
   [self setColor: &col  state: COLOR_BOTH];
 }
 
@@ -279,6 +276,8 @@
   CLAMP(s)
   CLAMP(b)
   gsMakeColor(&col, hsb_colorspace, h, s, b, 0);
+  // Keep the old alpha value
+  col.field[AINDEX] = fillColor.field[AINDEX];
   [self setColor: &col state: COLOR_BOTH];
 }
 
@@ -289,6 +288,8 @@
   CLAMP(g)
   CLAMP(b)
   gsMakeColor(&col, rgb_colorspace, r, g, b, 0);
+  // Keep the old alpha value
+  col.field[AINDEX] = fillColor.field[AINDEX];
   [self setColor: &col state: COLOR_BOTH];
 }
 
@@ -299,6 +300,8 @@
 
   ASSIGN(fillColorS, spaceref);
   gsMakeColor(&col, rgb_colorspace, 0, 0, 0, 0);
+  // Keep the old alpha value
+  col.field[AINDEX] = fillColor.field[AINDEX];
   [self setColor: &col state: COLOR_FILL];
 }
 
@@ -308,6 +311,8 @@
 
   ASSIGN(strokeColorS, spaceref);
   gsMakeColor(&col, rgb_colorspace, 0, 0, 0, 0);
+  // Keep the old alpha value
+  col.field[AINDEX] = fillColor.field[AINDEX];
   [self setColor: &col state: COLOR_STROKE];
 }
 
@@ -325,6 +330,7 @@
       DPS_ERROR(DPSundefined, @"No fill colorspace defined, assume DeviceRGB");
       gsMakeColor(&dcolor, rgb_colorspace, values[0], values[1], 
                   values[2], values[3]);
+      dcolor.field[AINDEX] = values[4];
     }
   else 
     {
@@ -351,6 +357,7 @@
       DPS_ERROR(DPSundefined, @"No stroke colorspace defined, assume DeviceRGB");
       gsMakeColor(&dcolor, rgb_colorspace, values[0], values[1], 
                   values[2], values[3]);
+      dcolor.field[AINDEX] = values[4];
     }
   else 
     {
@@ -638,13 +645,12 @@ typedef enum {
   if (ctm)
     [ctm makeIdentityMatrix];
   else
-    ctm = [[NSAffineTransform allocWithZone: GSObjCZone(self)] init];
+    ctm = [[NSAffineTransform allocWithZone: [self zone]] init];
 
    /* Initialize colors. By default the same color is used for filling and 
      stroking unless fill and/or stroke color is set explicitly */
   gsMakeColor(&fillColor, gray_colorspace, 0, 0, 0, 0);
   fillColor.field[AINDEX] = 1.0;
-  strokeColor.field[AINDEX] = 1.0;
   [self setColor: &fillColor state: COLOR_BOTH];
 
   charSpacing = 0;
@@ -652,7 +658,7 @@ typedef enum {
   if (textCtm)
     [textCtm makeIdentityMatrix];
   else
-    textCtm = [[NSAffineTransform allocWithZone: GSObjCZone(self)] init];
+    textCtm = [[NSAffineTransform allocWithZone: [self zone]] init];
 }
 
 - (void)DPScurrentflat: (float *)flatness 
@@ -694,7 +700,7 @@ typedef enum {
     }
 
   // This is rather slow, but it is not used very often
-  ictm = [ctm copyWithZone: GSObjCZone(self)];
+  ictm = [ctm copyWithZone: [self zone]];
   [ictm invert];
   user = [ictm transformPoint: [path currentPoint]];
   RELEASE(ictm);
@@ -941,7 +947,7 @@ typedef enum {
 {
   // This is rather slow, but it is not used very often
   NSBezierPath *newPath = [path copy];
-  NSAffineTransform *ictm = [ctm copyWithZone: GSObjCZone(self)];
+  NSAffineTransform *ictm = [ctm copyWithZone: [self zone]];
 
   [ictm invert];
   [newPath transformUsingAffineTransform: ictm];
@@ -1160,6 +1166,7 @@ typedef enum {
 
   inverse = [matrix copy];
   [inverse invert];
+  ts = [inverse transformStruct];
 
   rect = [function affectedRect];
   iwidth = rect.size.width;
@@ -1220,3 +1227,121 @@ typedef enum {
 @end
 
 
+@implementation GSGState (PatternColor)
+
+- (void *) saveClip
+{
+  [self subclassResponsibility: _cmd];
+  return NULL;
+}
+
+- (void) restoreClip: (void *)savedClip
+{
+  [self subclassResponsibility: _cmd];
+}
+
+- (void) _fillRect: (NSRect)rect withPattern: (NSImage*)color_pattern
+{
+  NSSize size;
+  float x;
+  float y;
+  NSAffineTransform *ictm;
+
+  // The coordinates we get here are already in device space,
+  // but compositeToPoint needs user space coordinates
+  ictm = [ctm copyWithZone: [self zone]];
+  [ictm invert];
+
+  size = [pattern size];
+  y = floor(NSMinY(rect) / size.height) * size.height;
+  while (y < NSMaxY(rect))
+    {
+      x = floor(NSMinX(rect) / size.width) * size.width;
+      while (x < NSMaxX(rect))
+        {
+          NSPoint p = NSMakePoint(x, y);
+          
+          p = [ictm transformPoint: p];
+	  [color_pattern compositeToPoint: p
+                         operation: NSCompositeSourceOver];
+	  x += size.width;
+        }
+      y += size.height;
+    }
+  RELEASE(ictm);
+}
+
+- (void) fillRect: (NSRect)rect withPattern: (NSImage*)color_pattern
+{
+  NSBezierPath *oldPath = path;
+  void *oldClip;
+
+  oldClip = [self saveClip];
+  path = [NSBezierPath bezierPathWithRect: rect];
+  [self DPSclip];
+
+  [self _fillRect: rect withPattern: color_pattern];
+
+  [self restoreClip: oldClip];
+  path = oldPath;
+}
+
+- (void) fillPath: (NSBezierPath*)fillPath withPattern: (NSImage*)color_pattern
+{
+  NSBezierPath *oldPath = path;
+  NSRect rect;
+  void *oldClip;
+
+  oldClip = [self saveClip];
+  rect = [fillPath bounds];
+  path = fillPath;
+  [self DPSclip];
+
+  [self _fillRect: rect withPattern: color_pattern];
+
+  [self restoreClip: oldClip];
+  path = oldPath;
+  [self DPSnewpath];
+}
+
+- (void) eofillPath: (NSBezierPath*)fillPath withPattern: (NSImage*)color_pattern
+{
+  NSBezierPath *oldPath = path;
+  NSRect rect;
+  void *oldClip;
+
+  oldClip = [self saveClip];
+  rect = [fillPath bounds];
+  path = fillPath;
+  [self DPSeoclip];
+
+  [self _fillRect: rect withPattern: color_pattern];
+
+  [self restoreClip: oldClip];
+  path = oldPath;
+  [self DPSnewpath];
+}
+
+@end
+
+@implementation GSGState (NSGradient)
+
+- (void) drawGradient: (NSGradient*)gradient
+           fromCenter: (NSPoint)startCenter
+               radius: (CGFloat)startRadius
+             toCenter: (NSPoint)endCenter 
+               radius: (CGFloat)endRadius
+              options: (NSUInteger)options
+{
+  [self subclassResponsibility: _cmd];
+}
+
+- (void) drawGradient: (NSGradient*)gradient
+            fromPoint: (NSPoint)startPoint
+              toPoint: (NSPoint)endPoint
+              options: (NSUInteger)options
+{
+  [self subclassResponsibility: _cmd];
+}
+
+@end
