@@ -48,6 +48,7 @@
 #include <AppKit/NSText.h>
 #include <AppKit/NSTextField.h>
 #include <AppKit/DPSOperators.h>
+#include <GNUstepGUI/GSTheme.h>
 
 #include "win32/WIN32Server.h"
 #include "win32/WIN32Geometry.h"
@@ -59,12 +60,15 @@
 #include <sys/file.h>
 #endif
 
+static BOOL _enableCallbacks = YES;
+
 static NSEvent *process_key_event(WIN32Server *svr, 
                                   HWND hwnd, WPARAM wParam, 
                                   LPARAM lParam, NSEventType eventType);
 static NSEvent *process_mouse_event(WIN32Server *svr, 
                                     HWND hwnd, WPARAM wParam, 
-                                    LPARAM lParam, NSEventType eventType);
+                                    LPARAM lParam, NSEventType eventType,
+                                    UINT uMsg);
 
 LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg, 
                              WPARAM wParam, LPARAM lParam);
@@ -95,7 +99,7 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 {
   MSG msg;
   WINBOOL bRet; 
-NSLog(@"Callback");
+//NSLog(@"Callback");
   while ((bRet = PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) != 0)
     { 
       if (msg.message == WM_QUIT)
@@ -247,48 +251,51 @@ NSLog(@"Callback");
 
   self = [super initWithAttributes: info];
 
-  if(self)
-  {
-    [self _initWin32Context];
-    [super initWithAttributes: info];
+  if (self)
+    {
+      [self _initWin32Context];
+      [super initWithAttributes: info];
 
-    [self setupRunLoopInputSourcesForMode: NSDefaultRunLoopMode]; 
-    [self setupRunLoopInputSourcesForMode: NSConnectionReplyMode]; 
-    [self setupRunLoopInputSourcesForMode: NSModalPanelRunLoopMode]; 
-    [self setupRunLoopInputSourcesForMode: NSEventTrackingRunLoopMode]; 
+      [self setupRunLoopInputSourcesForMode: NSDefaultRunLoopMode]; 
+      [self setupRunLoopInputSourcesForMode: NSConnectionReplyMode]; 
+      [self setupRunLoopInputSourcesForMode: NSModalPanelRunLoopMode]; 
+      [self setupRunLoopInputSourcesForMode: NSEventTrackingRunLoopMode]; 
 
-    [self setHandlesWindowDecorations: NO];
-    [self setUsesNativeTaskbar: YES];
+      [self setHandlesWindowDecorations: YES];
+      [self setUsesNativeTaskbar: YES];
 
-    { // Check user defaults
-      NSUserDefaults	*defs;
-      defs = [NSUserDefaults standardUserDefaults];
- 
-      if ([defs objectForKey: @"GSUseWMStyles"])
-        {
-          NSWarnLog(@"Usage of 'GSUseWMStyles' as user default option is deprecated. "
-                    @"This option will be ignored in future versions. "
-                    @"You should use 'GSBackHandlesWindowDecorations' option.");
-          [self setHandlesWindowDecorations: ![defs boolForKey: @"GSUseWMStyles"]];
-        }
-      if ([defs objectForKey: @"GSUsesWMTaskbar"])
-        {
-          NSWarnLog(@"Usage of 'GSUseWMTaskbar' as user default option is deprecated. "
-                    @"This option will be ignored in future versions. "
-                    @"You should use 'GSBackUsesNativeTaskbar' option.");
-          [self setUsesNativeTaskbar: [defs boolForKey: @"GSUseWMTaskbar"]];
-        }
+      [GSTheme theme];
+      { // Check user defaults
+	NSUserDefaults	*defs;
+	defs = [NSUserDefaults standardUserDefaults];
+   
+	if ([defs objectForKey: @"GSUseWMStyles"])
+	  {
+	    NSWarnLog(@"Usage of 'GSUseWMStyles' as user default option is deprecated. "
+		      @"This option will be ignored in future versions. "
+		      @"You should use 'GSBackHandlesWindowDecorations' option.");
+	    [self setHandlesWindowDecorations: ![defs boolForKey: @"GSUseWMStyles"]];
+	  }
+	if ([defs objectForKey: @"GSUsesWMTaskbar"])
+	  {
+	    NSWarnLog(@"Usage of 'GSUseWMTaskbar' as user default option is deprecated. "
+		      @"This option will be ignored in future versions. "
+		      @"You should use 'GSBackUsesNativeTaskbar' option.");
+	    [self setUsesNativeTaskbar: [defs boolForKey: @"GSUseWMTaskbar"]];
+	  }
 
-      if ([defs objectForKey: @"GSBackHandlesWindowDecorations"])
-        {
-          [self setHandlesWindowDecorations: [defs boolForKey: @"GSBackHandlesWindowDecorations"]];
-        } 
-      if ([defs objectForKey: @"GSBackUsesNativeTaskbar"])
-        {
-          [self setUsesNativeTaskbar: [defs boolForKey: @"GSUseNativeTaskbar"]];
-        }
+	if ([defs objectForKey: @"GSBackHandlesWindowDecorations"])
+	  {
+	    [self setHandlesWindowDecorations:
+	      [defs boolForKey: @"GSBackHandlesWindowDecorations"]];
+	  } 
+	if ([defs objectForKey: @"GSBackUsesNativeTaskbar"])
+	  {
+	    [self setUsesNativeTaskbar:
+	      [defs boolForKey: @"GSUseNativeTaskbar"]];
+	  }
+      }
     }
-  }
   return self;
 }
 
@@ -480,7 +487,7 @@ NSLog(@"Callback");
       estyle = WS_EX_TOOLWINDOW;
     } 
 
-   return estyle;
+  return estyle;
 }
 
 - (void) resizeBackingStoreFor: (HWND)hwnd
@@ -541,10 +548,12 @@ NSLog(@"Callback");
 
   [self setFlagsforEventLoop: hwnd];
  
+//NSLog(@"event %u", uMsg);
   switch (uMsg) 
     { 
       case WM_SIZING: 
-        [self decodeWM_SIZINGParams: hwnd : wParam : lParam];
+        return [self decodeWM_SIZINGParams: hwnd : wParam : lParam];
+	break;
       case WM_NCCREATE: 
         return [self decodeWM_NCCREATEParams: wParam : lParam : hwnd];
         break;
@@ -558,9 +567,9 @@ NSLog(@"Callback");
         if ([self handlesWindowDecorations])
           [self decodeWM_NCPAINTParams: wParam : lParam : hwnd]; 
         break;
-     //case WM_SHOWWINDOW: 
+      case WM_SHOWWINDOW: 
       //[self decodeWM_SHOWWINDOWParams: wParam : lParam : hwnd]; 
-      //break;
+        break;
       case WM_NCDESTROY: 
         [self decodeWM_NCDESTROYParams: wParam : lParam : hwnd]; 
         break;
@@ -657,9 +666,13 @@ NSLog(@"Callback");
       case WM_COMMAND: 
         [self decodeWM_COMMANDParams: wParam : lParam : hwnd];
         break;
-      case WM_SYSKEYDOWN: 
+      case WM_SYSKEYDOWN:  //KEYBOARD
+        NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "SYSKEYDOWN", hwnd);
+        ev = process_key_event(self, hwnd, wParam, lParam, NSKeyDown);
         break;
-      case WM_SYSKEYUP: 
+      case WM_SYSKEYUP:  //KEYBOARD
+        NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "SYSKEYUP", hwnd);
+        ev = process_key_event(self, hwnd, wParam, lParam, NSKeyUp);
         break;
       case WM_SYSCOMMAND: 
         [self decodeWM_SYSCOMMANDParams: wParam : lParam : hwnd];
@@ -697,45 +710,50 @@ NSLog(@"Callback");
         break;
       case WM_MOUSEMOVE: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "MOUSEMOVE", hwnd);
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSMouseMoved);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSMouseMoved, uMsg);
         break;
       case WM_LBUTTONDOWN: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "LBUTTONDOWN", hwnd);
         //[self decodeWM_LBUTTONDOWNParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd];
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSLeftMouseDown);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSLeftMouseDown, uMsg);
         break;
       case WM_LBUTTONUP: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "LBUTTONUP", hwnd);
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSLeftMouseUp);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSLeftMouseUp, uMsg);
         break;
       case WM_LBUTTONDBLCLK: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "LBUTTONDBLCLK", hwnd);
         break;
       case WM_MBUTTONDOWN: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "MBUTTONDOWN", hwnd);
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSOtherMouseDown);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSOtherMouseDown, uMsg);
         break;
       case WM_MBUTTONUP: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "MBUTTONUP", hwnd);
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSOtherMouseUp);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSOtherMouseUp, uMsg);
         break;
       case WM_MBUTTONDBLCLK: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "MBUTTONDBLCLK", hwnd);
         break;
       case WM_RBUTTONDOWN: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "RBUTTONDOWN", hwnd);
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSRightMouseDown);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSRightMouseDown, uMsg);
         break;
       case WM_RBUTTONUP: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "RBUTTONUP", hwnd);
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSRightMouseUp);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSRightMouseUp, uMsg);
         break;
       case WM_RBUTTONDBLCLK: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "RBUTTONDBLCLK", hwnd);
         break;
+      case WM_MOUSEHWHEEL: //MOUSE
+        NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "MOUSEHWHEEL", hwnd);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSScrollWheel, uMsg);
+        break;
+	
       case WM_MOUSEWHEEL: //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "MOUSEWHEEL", hwnd);
-        ev = process_mouse_event(self, hwnd, wParam, lParam, NSScrollWheel);
+        ev = process_mouse_event(self, hwnd, wParam, lParam, NSScrollWheel, uMsg);
         break;
 	
       case WM_KEYDOWN:  //KEYBOARD
@@ -867,7 +885,7 @@ NSLog(@"Callback");
               type, style, screen);
   NSDebugLLog(@"WTrace", @"         device frame: %d, %d, %d, %d", 
               r.left, r.top, r.right - r.left, r.bottom - r.top);
-  hwnd = CreateWindowEx(estyle, 
+  hwnd = CreateWindowEx(estyle | WS_EX_LAYERED, 
                         "GNUstepWindowClass", 
                         "GNUstepWindow", 
                         wstyle, 
@@ -881,6 +899,8 @@ NSLog(@"Callback");
                         (void*)type);
   NSDebugLLog(@"WTrace", @"         num/handle: %d", hwnd);
 
+  SetLayeredWindowAttributes(hwnd, 0, 255, LWA_ALPHA);
+
   [self _setWindowOwnedByServer: (int)hwnd];
   return (int)hwnd;
 }
@@ -991,6 +1011,7 @@ NSLog(@"Callback");
   int		foreground = 0;
   int		otherLevel;
   int		level;
+  NSWindow *window = GSWindowWithNumber(winNum);
 
   NSDebugLLog(@"WTrace", @"orderwindow: %d : %d : %d", op, otherWin, winNum);
 
@@ -1016,11 +1037,6 @@ NSLog(@"Callback");
         }
     }
 
-  flag = SW_SHOW;
-  if (IsIconic((HWND)winNum))
-    flag = SW_RESTORE;
-  ShowWindow((HWND)winNum, flag); 
-
   if (op == NSWindowOut)
     {
       SetWindowLong((HWND)winNum, OFF_ORDERED, 0);
@@ -1028,6 +1044,22 @@ NSLog(@"Callback");
         SWP_HIDEWINDOW | SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER);
       return;
     }
+
+  if (![window canBecomeMainWindow] && ![window canBecomeKeyWindow]) 
+    {   // Bring front, but do not activate, eg - tooltips
+    flag = SW_SHOWNA; 
+    ShowWindow((HWND)winNum, flag);
+    }
+  else 
+    {
+    flag = SW_SHOW;
+    
+    if (IsIconic((HWND)winNum))
+      flag = SW_RESTORE;
+    
+    ShowWindow((HWND)winNum, flag); 
+    }
+
   SetWindowLong((HWND)winNum, OFF_ORDERED, 1);
   level = GetWindowLong((HWND)winNum, OFF_LEVEL);
 
@@ -1165,6 +1197,12 @@ NSLog(@"Callback");
   SetWindowPos((HWND)winNum, (HWND)otherWin, 0, 0, 0, 0, 
     SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
 
+  if (otherWin == (int)HWND_TOP)
+    {
+      _enableCallbacks = NO;
+	  SetForegroundWindow((HWND)winNum);
+      _enableCallbacks = YES;
+	}
   /* For debug log window stack.
    */
   if (GSDebugSet(@"WTrace") == YES)
@@ -1208,7 +1246,7 @@ NSLog(@"Callback");
   p = GSWindowOriginToMS((HWND)winNum, loc);
 
   SetWindowPos((HWND)winNum, NULL, p.x, p.y, 0, 0, 
-               SWP_NOZORDER | SWP_NOSIZE);
+               SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);
 }
 
 - (void) placewindow: (NSRect)frame : (int) winNum
@@ -1223,7 +1261,8 @@ NSLog(@"Callback");
   GetWindowRect((HWND)winNum, &r2);
 
   SetWindowPos((HWND)winNum, NULL, 
-    r.left, r.top, r.right - r.left, r.bottom - r.top, SWP_NOZORDER); 
+               r.left, r.top, r.right - r.left, r.bottom - r.top, 
+               SWP_NOZORDER | SWP_NOACTIVATE); 
 
   if ((win->useHDC)
       && (r.right - r.left != r2.right - r2.left)
@@ -1416,9 +1455,11 @@ NSLog(@"Callback");
     {
       return;
     }
-  if (state == GSTitleBarKey)
+  if (state == GSTitleBarMain)
     {
+      _enableCallbacks = NO;
       SetActiveWindow((HWND)winNum);
+      _enableCallbacks = YES;
     }
 }
 
@@ -1635,6 +1676,65 @@ process_char(WPARAM wParam, unsigned *eventModifierFlags)
     }
 }
 
+static unsigned int
+mask_for_keystate(BYTE *keyState)
+{
+  unsigned int eventFlags = 0;
+  NSUserDefaults *defs = [NSUserDefaults standardUserDefaults];
+  NSString *firstCommand = [defs stringForKey: @"GSFirstCommandKey"];
+  NSString *firstControl = [defs stringForKey: @"GSFirstControlKey"];
+  NSString *firstAlt = [defs stringForKey: @"GSFirstAlternateKey"];
+  NSString *secondCommand = [defs stringForKey: @"GSSecondCommandKey"];
+  NSString *secondControl = [defs stringForKey: @"GSSecondControlKey"];
+  NSString *secondAlt = [defs stringForKey: @"GSSecondAlternateKey"];
+
+  /* AltGr is mapped to right alt + left control */ 
+  if (keyState[VK_RCONTROL] & 128) // && !((keyState[VK_LCONTROL] & 128) && (keyState[VK_RMENU] & 128))) 
+    {
+      if([@"Control_R" isEqualToString: firstAlt] ||
+	 [@"Control_R" isEqualToString: secondAlt])
+	eventFlags |= NSAlternateKeyMask;
+      else if([@"Control_R" isEqualToString: firstCommand] ||
+	      [@"Control_R" isEqualToString: secondCommand])
+	eventFlags |= NSCommandKeyMask;
+      else
+	eventFlags |= NSControlKeyMask;
+    }
+
+  if (keyState[VK_SHIFT] & 128)
+    eventFlags |= NSShiftKeyMask;
+  if (keyState[VK_CAPITAL] & 128)
+    eventFlags |= NSShiftKeyMask;
+
+  if (keyState[VK_MENU] & 128)
+    {
+      if([@"Alt_R" isEqualToString: firstControl] ||
+	 [@"Alt_R" isEqualToString: secondControl])
+	eventFlags |= NSControlKeyMask;
+      else if([@"Alt_R" isEqualToString: firstCommand] ||
+	      [@"Alt_R" isEqualToString: secondCommand])
+	eventFlags |= NSCommandKeyMask;
+      else
+	eventFlags |= NSAlternateKeyMask;
+    }
+
+  if (keyState[VK_HELP] & 128)
+    eventFlags |= NSHelpKeyMask;
+
+  if ((keyState[VK_LCONTROL] & 128) || (keyState[VK_RWIN] & 128))
+    {
+      if([@"Control_L" isEqualToString: firstAlt] ||
+	 [@"Control_L" isEqualToString: secondAlt])
+	eventFlags |= NSAlternateKeyMask;
+      else if([@"Control_L" isEqualToString: firstControl] ||
+	      [@"Control_L" isEqualToString: secondControl])
+	eventFlags |= NSControlKeyMask;
+      else
+	eventFlags |= NSCommandKeyMask;
+    }
+  return eventFlags;
+}
+
 static NSEvent*
 process_key_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam, 
 		  NSEventType eventType)
@@ -1667,21 +1767,7 @@ process_key_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
   time = ltime / 1000;
 
   GetKeyboardState(keyState);
-  eventFlags = 0;
-  /* AltGr is mapped to right alt + left control */ 
-  if ((keyState[VK_CONTROL] & 128) && !((keyState[VK_LCONTROL] & 128) && (keyState[VK_RMENU] & 128))) 
-    eventFlags |= NSControlKeyMask;
-  if (keyState[VK_SHIFT] & 128)
-    eventFlags |= NSShiftKeyMask;
-  if (keyState[VK_CAPITAL] & 128)
-    eventFlags |= NSShiftKeyMask;
-  if (keyState[VK_MENU] & 128)
-    eventFlags |= NSAlternateKeyMask;
-  if (keyState[VK_HELP] & 128)
-    eventFlags |= NSHelpKeyMask;
-  if ((keyState[VK_LWIN] & 128) || (keyState[VK_RWIN] & 128))
-    eventFlags |= NSCommandKeyMask;
-
+  eventFlags = mask_for_keystate(keyState);
 
   switch(wParam)
     {
@@ -1718,6 +1804,13 @@ process_key_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
     }
   else
     {
+      BYTE blankKeyState[256];
+      int i = 0;
+
+      // initialize blank key state array....
+      for(i = 0; i < 256; i++)
+	blankKeyState[i] = 0;
+
       scan = ((lParam >> 16) & 0xFF);
       //NSLog(@"Got key code %d %d", scan, wParam);
       result = ToUnicode(wParam, scan, keyState, unicode, 5, 0);
@@ -1727,27 +1820,33 @@ process_key_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
 	  // A non spacing accent key was found, we still try to use the result 
 	  result = 1;
 	}
-      keys = [NSString  stringWithCharacters: unicode  length: result];
-      // Now switch modifiers off
-      keyState[VK_LCONTROL] = 0;
-      keyState[VK_RCONTROL] = 0;
-      keyState[VK_LMENU] = 0;
-      keyState[VK_RMENU] = 0;
-      result = ToUnicode(wParam, scan, keyState, unicode, 5, 0);
+      keys = [NSString  stringWithCharacters: unicode length: result];
+
+      // Now get the characters with a blank keyboard state so that
+      // no modifiers are applied.
+      result = ToUnicode(wParam, scan, blankKeyState, unicode, 5, 0);
       //NSLog(@"To Unicode resulted in %d with %d", result, unicode[0]);
       if (result == -1)
 	{
 	  // A non spacing accent key was found, we still try to use the result 
 	  result = 1;
 	}
+      
       ukeys = [NSString  stringWithCharacters: unicode  length: result];
     }
-
+  if (eventFlags & NSShiftKeyMask)
+    ukeys = [ukeys uppercaseString];
+  
+  // key events should go to the key window if we have one (Windows' focus window isn't always appropriate)
+  int windowNumber = [[NSApp keyWindow] windowNumber];
+  if (windowNumber == 0)
+    windowNumber  = (int)hwnd;
+	
   event = [NSEvent keyEventWithType: eventType
 			   location: eventLocation
 		      modifierFlags: eventFlags
 			  timestamp: time
-		       windowNumber: (int)hwnd
+		       windowNumber: windowNumber
 			    context: gcontext
 			 characters: keys
 		   charactersIgnoringModifiers: ukeys
@@ -1759,22 +1858,51 @@ process_key_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
 
 static NSEvent*
 process_mouse_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam, 
-		    NSEventType eventType)
+		    NSEventType eventType, UINT uMsg)
 {
   NSEvent *event;
   NSPoint eventLocation;
+  static NSPoint lastLocation = {0.0, 0.0};
   unsigned int eventFlags;
   NSTimeInterval time;
   LONG ltime;
   DWORD tick;
   NSGraphicsContext *gcontext;
-  short deltaY = 0;
+  float deltaX = 0.0, deltaY = 0.0;
   static int clickCount = 1;
   static LONG lastTime = 0;
+  int clientX, clientY;
+/*
+ * Occasionally the mouse down events are lost ... don't know why.
+ * So we track the mouse status  and simulate mouse down or up events
+ * if the button states appear to have changed when we get a move.
+ */
+  static BOOL lDown = NO;
+  static BOOL oDown = NO;
+  static BOOL rDown = NO;
 
   gcontext = GSCurrentContext();
-  eventLocation = MSWindowPointToGS(svr, hwnd,  GET_X_LPARAM(lParam), 
-				    GET_Y_LPARAM(lParam));
+
+/*
+ * Some events give screen coordinates - we must convert those to client
+ * coordinates.
+ */
+  if (eventType == NSScrollWheel)
+    {
+      POINT point;
+      point.x = GET_X_LPARAM(lParam);
+      point.y = GET_Y_LPARAM(lParam);
+      ScreenToClient(hwnd, &point);
+      clientX = point.x;
+      clientY = point.y;
+    }
+  else
+    {
+      clientX = GET_X_LPARAM(lParam);
+      clientY = GET_Y_LPARAM(lParam);
+    }
+
+  eventLocation = MSWindowPointToGS(svr, hwnd, clientX, clientY);
   ltime = GetMessageTime();
   time = ltime / 1000;
   tick = GetTickCount();
@@ -1799,28 +1927,97 @@ process_mouse_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
 
   if (eventType == NSScrollWheel)
     {
-      deltaY = GET_WHEEL_DELTA_WPARAM(wParam) / 120;
-      //NSLog(@"Scroll event with delat %d", deltaY);
+      float delta = GET_WHEEL_DELTA_WPARAM(wParam) / 120.0;
+      if (uMsg == WM_MOUSEWHEEL)
+        deltaY = delta;
+      else
+        deltaX = delta;
+      //NSLog(@"Scroll event with deltaX %f deltaY %f", deltaX, deltaY);
     }
   else if (eventType == NSMouseMoved)
     {
+      NSEvent	*e;
+      deltaX = eventLocation.x - lastLocation.x;
+      deltaY = -(eventLocation.y - lastLocation.y);
+
       if (wParam & MK_LBUTTON)
         { 
+	  if (lDown == NO)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSLeftMouseDown, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
           eventType = NSLeftMouseDragged;
         }
       else if (wParam & MK_RBUTTON)
         {
+	  if (lDown == YES)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSLeftMouseUp, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
+	  if (rDown == NO)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSRightMouseDown, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
           eventType = NSRightMouseDragged;
         }
       else if (wParam & MK_MBUTTON)
         {
+	  if (lDown == YES)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSLeftMouseUp, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
+	  if (rDown == YES)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSRightMouseUp, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
+	  if (oDown == NO)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSOtherMouseDown, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
           eventType = NSOtherMouseDragged;
         }
+      else
+	{
+	  if (lDown == YES)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSLeftMouseUp, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
+	  if (rDown == YES)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSRightMouseUp, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
+	  if (oDown == YES)
+	    {
+	      e = process_mouse_event(svr, hwnd, wParam, lParam,
+		NSOtherMouseUp, uMsg);
+	      [GSCurrentServer() postEvent: e atStart: NO];
+	    }
+	}
     }
   else if ((eventType == NSLeftMouseDown)
     || (eventType == NSRightMouseDown)
     || (eventType == NSOtherMouseDown))
     {
+      // It seems Windows generates duplicate mouse down events on first click in a window
+      if (ltime == lastTime) // duplicate event has identical time
+	return nil; // ignore it
+
       if (lastTime + GetDoubleClickTime() > ltime)
         {
           clickCount += 1;
@@ -1830,7 +2027,32 @@ process_mouse_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
           clickCount = 1;
           lastTime = ltime;
         }
+      SetCapture(hwnd); // capture the mouse to get mouse moved events outside of window
     }
+  else if ( ((eventType == NSLeftMouseUp)
+    || (eventType == NSRightMouseUp)
+    || (eventType == NSOtherMouseUp)) 
+	&& !((wParam & MK_LBUTTON) || (wParam & MK_MBUTTON) || (wParam & MK_RBUTTON)) )
+	{
+	  ReleaseCapture(); // release capture when all mouse buttons are up
+	}
+
+  if (eventType == NSLeftMouseDown) lDown = YES;
+  if (eventType == NSRightMouseDown) rDown = YES;
+  if (eventType == NSOtherMouseDown) oDown = YES;
+  if (eventType == NSLeftMouseUp) lDown = NO;
+  if (eventType == NSRightMouseUp) rDown = NO;
+  if (eventType == NSOtherMouseUp) oDown = NO;
+  
+  if (eventType == NSMouseMoved) {
+    static HWND lastHwnd = 0;
+	if (hwnd == lastHwnd && NSEqualPoints(eventLocation, lastLocation))
+	  return nil; // mouse hasn't actually moved -- don't generate another event
+	// record window and location of this event, to check next mouseMoved event against
+	lastHwnd = hwnd;	
+  }
+
+  lastLocation = eventLocation;
 
   event = [NSEvent mouseEventWithType: eventType
 			     location: eventLocation
@@ -1842,10 +2064,10 @@ process_mouse_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
 			   clickCount: clickCount
 			     pressure: 1.0
 			 buttonNumber: 0 /* FIXME */
-			       deltaX: 0.
+			       deltaX: deltaX
 			       deltaY: deltaY
 			       deltaZ: 0.];
-            
+  
   return event;
 }
 
@@ -1854,6 +2076,11 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 			     WPARAM wParam, LPARAM lParam)
 {
   WIN32Server	*ctxt = (WIN32Server *)GSCurrentServer();
+  
+  if(_enableCallbacks == NO)
+    {
+      return (LRESULT)NULL;
+    }
 
   return [ctxt windowEventProc: hwnd : uMsg : wParam : lParam];
 }
