@@ -25,6 +25,7 @@
 */
 
 #include <Foundation/NSCharacterSet.h>
+#include <Foundation/NSDebug.h>
 #include <Foundation/NSDictionary.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSArray.h>
@@ -43,7 +44,7 @@ NSString *win32_font_family(NSString *fontName);
 @implementation WIN32FontInfo
 
 - initWithFontName: (NSString*)name
-	    matrix: (const float *)fmatrix
+	    matrix: (const CGFloat *)fmatrix
 	screenFont: (BOOL)screenFont
 {
   if (screenFont)
@@ -75,7 +76,7 @@ NSString *win32_font_family(NSString *fontName);
   [super dealloc];
 }
 
-- (float) widthOfString: (NSString*)string
+- (CGFloat) widthOfString: (NSString*)string
 {
   SIZE size;
   HDC hdc;
@@ -451,7 +452,7 @@ NSLog(@"No glyph for U%d", c);
   if (range.length)
     logfont.lfItalic = 1; 
 
-  logfont.lfQuality = ANTIALIASED_QUALITY;
+  logfont.lfQuality = DEFAULT_QUALITY;
   wcsncpy(logfont.lfFaceName,
     (const unichar*)[familyName cStringUsingEncoding: NSUnicodeStringEncoding],
     LF_FACESIZE);
