@@ -31,7 +31,12 @@
 
 // Define this so we pick up AlphaBlend, when loading windows.h
 #ifdef USE_ALPHABLEND
+#if	!defined(WINVER)
 #define WINVER 0x0500
+#elif (WINVER < 0x0500)
+#undef	WINVER
+#define WINVER 0x0500
+#endif
 #endif
 
 #include <AppKit/NSAffineTransform.h>
@@ -111,6 +116,79 @@ RECT GSViewRectToWin(WIN32GState *s, NSRect r)
   return GSWindowRectToMS(s, r);
 }
 
+void* get_bits(HDC dc,
+	       int w,
+	       int h,
+	       HBITMAP *bitmap)
+{
+  void          *bits = NULL;
+  BITMAPINFO     info;
+  HDC            cDC;
+
+  info.bmiHeader.biSize = sizeof(BITMAPINFO);
+  info.bmiHeader.biWidth = w;
+  info.bmiHeader.biHeight = h;
+  info.bmiHeader.biPlanes = 1;
+  info.bmiHeader.biBitCount = 32;
+  info.bmiHeader.biCompression = BI_RGB;
+  info.bmiHeader.biSizeImage = 0;
+  info.bmiHeader.biXPelsPerMeter = 0;
+  info.bmiHeader.biYPelsPerMeter = 0;
+  info.bmiHeader.biClrUsed = 0;
+  info.bmiHeader.biClrImportant = 0;
+
+  if(!(cDC = CreateCompatibleDC(dc)))
+    {
+      NSLog(@"Could not create compatible DC");
+      return NULL;
+    }
+  
+  if(!(*bitmap = CreateDIBSection(dc, (LPBITMAPINFO)&info, 
+				  DIB_RGB_COLORS, &bits, 
+				  NULL, 0)))
+    {
+      NSLog(@"Could not create bit map from DC");
+      return NULL;
+    }
+  
+  SelectObject(cDC, *bitmap);
+  
+  return bits;
+}
+
+BOOL alpha_blend_source_over(HDC destDC, 
+			     HDC srcDC, 
+			     RECT rectFrom, 
+			     int x, int y, int w, int h, 
+			     float delta)
+{
+  BOOL success = YES;
+
+#ifdef USE_ALPHABLEND
+  // Use (0..1) fraction to set a (0..255) alpha constant value
+  BYTE SourceConstantAlpha = (BYTE)(delta * 255);
+  BLENDFUNCTION blendFunc
+    = {AC_SRC_OVER, 0, SourceConstantAlpha, AC_SRC_ALPHA};
+
+  /* There is actually a very real chance this could fail, even on 
+     computers that supposedly support it. It's not known why it
+     fails though... */
+  success = AlphaBlend(destDC,
+		       x, y, w, h,
+		       srcDC,
+		       rectFrom.left, rectFrom.top,
+		       w, h, blendFunc);
+  // #else
+  // HBITMAP    sbitmap;
+  // HBITMAP    dbitmap; 
+  // unsigned char *sbits = (unsigned char *)get_bits(srcDC,w,h,&sbitmap);
+  // unsigned char *dbits = (unsigned char *)get_bits(destDC,w,h,&dbitmap);
+
+#endif
+
+  return success;
+}
+
 @interface WIN32GState (WinOps)
 - (void) setStyle: (HDC)hDC;
 - (void) restoreStyle: (HDC)hDC;
@@ -205,6 +283,7 @@ RECT GSViewRectToWin(WIN32GState *s, NSRect r)
 
     [source->ctm boundingRectFor: sourceRect result: &newRect];
     rectFrom = GSWindowRectToMS(source, newRect);
+    y += (sourceRect.size.height - newRect.size.height); // adjust location for scaled source
   }
   h = rectFrom.bottom - rectFrom.top;
   w = rectFrom.right - rectFrom.left;
@@ -233,22 +312,13 @@ RECT GSViewRectToWin(WIN32GState *s, NSRect r)
     {
     case NSCompositeSourceOver:
       {
-#ifdef USE_ALPHABLEND
-        // Use (0..1) fraction to set a (0..255) alpha constant value
-        BYTE SourceConstantAlpha = (BYTE)(delta * 255);
-        BLENDFUNCTION blendFunc
-            = {AC_SRC_OVER, 0, SourceConstantAlpha, AC_SRC_ALPHA};
-        success = AlphaBlend(hDC,
-                             x, y, w, h,
-                             sourceDC,
-                             rectFrom.left, rectFrom.top,
-                             w, h, blendFunc);
-        /* There is actually a very real chance this could fail, even on 
-           computers that supposedly support it. It's not known why it
-           fails though... */
-        if (success)
-            break;
-#endif
+	success = alpha_blend_source_over(hDC, 
+					  sourceDC, 
+					  rectFrom, 
+					  x, y, w, h, 
+					  delta);
+	if (success)
+	  break;
       }
     case NSCompositeCopy:
       {
@@ -282,34 +352,11 @@ RECT GSViewRectToWin(WIN32GState *s, NSRect r)
   [source releaseHDC: sourceDC];
 }
 
-- (void) compositeGState: (GSGState *)source 
-                fromRect: (NSRect)aRect
-                 toPoint: (NSPoint)aPoint
-                      op: (NSCompositingOperation)op
-{
-  [self compositeGState: (WIN32GState *) source
-	       fromRect: aRect
-	        toPoint: aPoint
-	             op: op
-	       fraction: 1.0];
-}
-
-- (void) dissolveGState: (GSGState *)source
-	       fromRect: (NSRect)aRect
-		toPoint: (NSPoint)aPoint 
-		  delta: (float)delta
-{
-  [self compositeGState: (WIN32GState *) source
-	       fromRect: aRect
-	        toPoint: aPoint
-	             op: NSCompositeSourceOver
-	       fraction: delta];
-}
-
 - (void) compositerect: (NSRect)aRect
                     op: (NSCompositingOperation)op
 {
   float gray;
+  BOOL success = NO;
 
   // FIXME: This is taken from the xlib backend
   [self DPScurrentgray: &gray];
@@ -747,15 +794,24 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
       switch (drawType)
 	{
 	case path_stroke:
-	  StrokePath(hDC);
+	  if (strokeColor.field[AINDEX] != 0.0)
+	    {
+	      StrokePath(hDC);
+	    }
 	  break;
 	case path_eofill:
-	  SetPolyFillMode(hDC, ALTERNATE);
-	  FillPath(hDC);
+	  if (fillColor.field[AINDEX] != 0.0)
+	    {
+	      SetPolyFillMode(hDC, ALTERNATE);
+	      FillPath(hDC);
+	    }
 	  break;
 	case path_fill:
-	  SetPolyFillMode(hDC, WINDING);
-	  FillPath(hDC);
+	  if (fillColor.field[AINDEX] != 0.0)
+	    {
+	      SetPolyFillMode(hDC, WINDING);
+	      FillPath(hDC);
+	    }
 	  break;
 	case path_eoclip:
 	  {
@@ -764,10 +820,10 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 	    SetPolyFillMode(hDC, ALTERNATE);
 	    region = PathToRegion(hDC);
 	    if (clipRegion)
-	    {
-	      CombineRgn(clipRegion, clipRegion, region, RGN_AND);
-	      DeleteObject(region);
-	    }
+              {
+                CombineRgn(clipRegion, clipRegion, region, RGN_AND);
+                DeleteObject(region);
+              }
 	    else
 	      {
 		clipRegion = region;
@@ -781,10 +837,10 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 	    SetPolyFillMode(hDC, WINDING);
 	    region = PathToRegion(hDC);
 	    if (clipRegion)
-	    {
-	      CombineRgn(clipRegion, clipRegion, region, RGN_AND);
-	      DeleteObject(region);
-	    }
+              {
+                CombineRgn(clipRegion, clipRegion, region, RGN_AND);
+                DeleteObject(region);
+              }
 	    else
 	      {
 		clipRegion = region;
@@ -819,11 +875,22 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 
 - (void)DPSeofill 
 {
+  if (pattern != nil)
+    {
+      [self eofillPath: path withPattern: pattern];
+      return;
+    }
   [self _paintPath: path_eofill];
 }
 
 - (void)DPSfill 
 {
+  if (pattern != nil)
+    {
+      [self fillPath: path withPattern: pattern];
+      return;
+    }
+
   [self _paintPath: path_fill];
 }
 
@@ -1127,3 +1194,224 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 }
 
 @end
+
+
+@implementation WIN32GState (PatternColor)
+
+- (void *) saveClip
+{
+  if (clipRegion)
+    {
+      HRGN newClipRegion;
+
+      newClipRegion = CreateRectRgn(0, 0, 1, 1);
+      CombineRgn(newClipRegion, clipRegion, NULL, RGN_COPY);
+
+      return newClipRegion;
+    }
+  return clipRegion;
+}
+
+- (void) restoreClip: (void *)savedClip
+{
+  if (clipRegion)
+    {
+      DeleteObject(clipRegion);
+    }
+  clipRegion = savedClip;
+}
+
+@end
+
+@implementation WIN32GState (ReadRect)
+
+- (NSDictionary *) GSReadRect: (NSRect)r
+{
+  NSMutableDictionary *dict;
+  NSAffineTransform *matrix;
+  double x, y;
+  NSMutableData *data;
+  unsigned char *cdata;
+  unsigned char *bits;
+  int i = 0;
+  HDC hDC;
+  HDC hdcMemDC = NULL;
+  HBITMAP hbitmap = NULL;
+  BITMAP bmpCopied;
+  HGDIOBJ old;
+  RECT rcClient;
+  DWORD dwBmpSize;
+  HANDLE hDIB;
+  LPBITMAPV4HEADER lpbi;
+
+  if (window == NULL)
+    {
+      NSLog(@"No window in GSReadRect");
+      return nil;
+    }
+
+  r = [ctm rectInMatrixSpace: r];
+  x = NSWidth(r);
+  y = NSHeight(r);
+
+  dict = [NSMutableDictionary dictionary];
+  [dict setObject: NSDeviceRGBColorSpace forKey: @"ColorSpace"];
+
+  [dict setObject: [NSNumber numberWithUnsignedInt: 8]
+	   forKey: @"BitsPerSample"];
+  [dict setObject: [NSNumber numberWithUnsignedInt: 32]
+	   forKey: @"Depth"];
+  [dict setObject: [NSNumber numberWithUnsignedInt: 4] 
+           forKey: @"SamplesPerPixel"];
+  [dict setObject: [NSNumber numberWithUnsignedInt: 1]
+           forKey: @"HasAlpha"];
+
+  matrix = [self GSCurrentCTM];
+  [matrix translateXBy: -r.origin.x - offset.x 
+		   yBy: r.origin.y + NSHeight(r) - offset.y];
+  [dict setObject: matrix forKey: @"Matrix"];
+
+  hDC = GetDC((HWND)window);
+  if (!hDC)
+    {
+      NSLog(@"No DC for window %d in GSReadRect. Error %d", 
+	(int)window, GetLastError());
+      return nil;
+    }
+
+  // Create a compatible DC which is used in a BitBlt from the window DC
+  hdcMemDC = CreateCompatibleDC(hDC); 
+  if (!hdcMemDC)
+    {
+      NSLog(@"No Compatible DC for window %d in GSReadRect. Error %d", 
+	(int)window, GetLastError());
+      ReleaseDC((HWND)window, hDC);
+      return nil;
+    }
+
+  ReleaseDC((HWND)window, hDC);
+  hDC = [self getHDC];
+
+  // Get the client area for size calculation
+  rcClient = GSWindowRectToMS(self, r);
+
+  // Create a compatible bitmap from the Window DC
+  hbitmap = CreateCompatibleBitmap(hDC, rcClient.right - rcClient.left, 
+  rcClient.bottom - rcClient.top);
+  if (!hbitmap)
+    {
+      NSLog(@"No Compatible bitmap for window %d in GSReadRect. Error %d", 
+	(int)window, GetLastError());
+      ReleaseDC((HWND)window, hdcMemDC);
+      [self releaseHDC: hDC];
+      return nil;
+    }
+
+  // Select the compatible bitmap into the compatible memory DC.
+  old = SelectObject(hdcMemDC, hbitmap);
+  if (!old)
+    {
+      NSLog(@"SelectObject failed for window %d in GSReadRect. Error %d", 
+	(int)window, GetLastError());
+      DeleteObject(hbitmap);
+      ReleaseDC((HWND)window, hdcMemDC);
+      [self releaseHDC: hDC];
+      return nil;
+    }
+
+  // Bit block transfer into our compatible memory DC.
+  if (!BitBlt(hdcMemDC, 0, 0, 
+    rcClient.right - rcClient.left, 
+    rcClient.bottom - rcClient.top, 
+    hDC, 
+    0, 0,
+    SRCCOPY))
+    {
+      NSLog(@"BitBlt failed for window %d in GSReadRect. Error %d", 
+	(int)window, GetLastError());
+      DeleteObject(hbitmap);
+      ReleaseDC((HWND)window, hdcMemDC);
+      [self releaseHDC: hDC];
+      return nil;
+    }
+
+  // Get the BITMAP from the HBITMAP
+  GetObject(hbitmap, sizeof(BITMAP), &bmpCopied);
+
+  dwBmpSize = bmpCopied.bmWidth * 4 * bmpCopied.bmHeight;
+
+  hDIB = GlobalAlloc(GHND, dwBmpSize + sizeof(BITMAPV4HEADER)); 
+  lpbi = (LPBITMAPV4HEADER)GlobalLock(hDIB);    
+  lpbi->bV4Size = sizeof(BITMAPV4HEADER);
+  lpbi->bV4V4Compression = BI_BITFIELDS;
+  lpbi->bV4BlueMask = 0x000000FF;
+  lpbi->bV4GreenMask = 0x0000FF00;
+  lpbi->bV4RedMask = 0x00FF0000;
+  lpbi->bV4AlphaMask = 0xFF000000;
+  lpbi->bV4Width = bmpCopied.bmWidth;    
+  lpbi->bV4Height = bmpCopied.bmHeight;  
+  lpbi->bV4Planes = 1;    
+  lpbi->bV4BitCount = 32;    
+  lpbi->bV4SizeImage = 0;  
+  lpbi->bV4XPelsPerMeter = 0;    
+  lpbi->bV4YPelsPerMeter = 0;    
+  lpbi->bV4ClrUsed = 0;    
+  lpbi->bV4ClrImportant = 0;
+  bits = (unsigned char *)lpbi + sizeof(BITMAPV4HEADER);
+
+  // Gets the "bits" from the bitmap and copies them into a buffer 
+  // which is pointed to by lpbi
+  if (GetDIBits(hdcMemDC, hbitmap, 0, (UINT)bmpCopied.bmHeight, bits,
+    (BITMAPINFOHEADER *)lpbi, DIB_RGB_COLORS) == 0)
+    {
+      NSLog(@"GetDIBits failed for window %d in GSReadRect. Error %d", 
+	(int)window, GetLastError());
+      GlobalUnlock(hDIB);    
+      GlobalFree(hDIB);
+      DeleteObject(hbitmap);
+      ReleaseDC((HWND)window, hdcMemDC);
+      [self releaseHDC: hDC];
+      return nil;
+    }
+
+  data = [NSMutableData dataWithLength: dwBmpSize];
+  if (data == nil)
+    {
+      GlobalUnlock(hDIB);    
+      GlobalFree(hDIB);
+      DeleteObject(hbitmap);
+      ReleaseDC((HWND)window, hdcMemDC);
+      [self releaseHDC: hDC];
+      return nil;
+    }
+
+  // Copy to data
+  cdata = [data mutableBytes];
+  while (i < dwBmpSize)
+    {
+      cdata[i+0] = bits[i+2];
+      cdata[i+1] = bits[i+1];
+      cdata[i+2] = bits[i+0];
+      cdata[i+3] = bits[i+3];
+      i += 4;
+    }
+
+
+  //Unlock and Free the DIB from the heap
+  GlobalUnlock(hDIB);    
+  GlobalFree(hDIB);
+
+  //Clean up
+  DeleteObject(hbitmap);
+  ReleaseDC((HWND)window, hdcMemDC);
+  [self releaseHDC: hDC];
+
+  [dict setObject: [NSValue valueWithSize: NSMakeSize(bmpCopied.bmWidth,
+  bmpCopied.bmHeight)]
+  forKey: @"Size"];
+  [dict setObject: data forKey: @"Data"];
+
+  return dict;
+}
+
+@end
