@@ -29,6 +29,7 @@
 #ifdef HAVE_GLX
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSException.h>
+#include <Foundation/NSDictionary.h>
 #include <GNUstepGUI/GSDisplayServer.h>
 #include <AppKit/NSView.h>
 #include <AppKit/NSWindow.h>
@@ -69,16 +70,21 @@
   XSetWindowAttributes window_attributes;
 
   self = [super init];
+
   if (!self)
-    return nil;
+    {
+      return nil;
+    }
 
   window = [view window];
-  NSAssert(window, @"request of an X window attachment on a view that is not on a NSWindow");
+  NSAssert(window, @"Request of an X window attachment on a view that is not\
+                     on a NSWindow");
 
   if ([view isRotatedOrScaledFromBase])
     {
       [NSException raise: NSInvalidArgumentException
-                   format: @"Cannot attach an Xwindow to a view that is rotated or scaled"];
+                   format: @"Cannot attach an Xwindow to a view that is\
+                           rotated or scaled"];
     }
   
   server = (XGServer *)GSServerForWindow(window);
@@ -113,11 +119,12 @@
   width = NSWidth(rect);
   height = NSHeight(rect);
 
-  window_attributes.border_pixel = 255;
+  window_attributes.border_pixel = 0;
   window_attributes.background_pixel = 0;
   window_attributes.colormap = XCreateColormap(win_info->display, 
                                                win_info->ident,
                                                xVisualInfo->visual, AllocNone);
+
   window_attributes.event_mask = StructureNotifyMask
                                | VisibilityChangeMask
                                | ExposureMask;
@@ -126,8 +133,8 @@
 
   xwindowid = XCreateWindow(win_info->display, win_info->ident,
                             x, y, width, height, 0, 
-                            xVisualInfo->depth, InputOutput, xVisualInfo->visual, 
-                            mask, &window_attributes);
+                            xVisualInfo->depth, InputOutput,
+                            xVisualInfo->visual, mask, &window_attributes);
 
   XMapWindow(win_info->display, xwindowid);
 
@@ -208,7 +215,8 @@
 
 - (void) dealloc
 {
-  NSDebugMLLog(@"GLX", @"deallocating");
+  NSDebugMLLog(@"GLX", @"Deallocating");
+
   [self detach];
   [super dealloc];
 }
@@ -219,30 +227,36 @@
 
   return AUTORELEASE(win);
 }
+
 @end
 
 //FIXME:
 //should be on per thread basis.
 static XGGLContext *currentGLContext;
 
-
 @implementation XGGLContext
 
 + (void)clearCurrentContext
 {
   MAKE_DISPLAY(dpy);
 
-  if (GSglxMinorVersion(dpy) >= 3)
+  if ([XGGLPixelFormat glxMinorVersion] >= 3)
     {
       if ( !glXMakeContextCurrent(dpy, None, None, NULL) )
-        NSDebugMLLog( @"GLX", @"Can not clear current GL context - Errror %u",
-                              glGetError() );
+        {
+          NSDebugMLLog( @"GLX", 
+                        @"Can not clear current GL context - Errror %u",
+                        glGetError() );
+        }
     }
   else
     {
       if ( !glXMakeCurrent(dpy, None, NULL) )
-        NSDebugMLLog( @"GLX", @"Can not clear current GL context - Errror %u",
-                              glGetError() );
+        {
+          NSDebugMLLog( @"GLX", 
+                        @"Can not clear current GL context - Errror %u",
+                        glGetError() );
+        }
     }
 
   currentGLContext = nil;
@@ -260,12 +274,16 @@ static XGGLContext *currentGLContext;
       MAKE_DISPLAY(dpy);
 
       if (currentGLContext == self)
-	      {
+	    {
           [XGGLContext clearCurrentContext];
         }
-      // FIXME:
-      //      glXDestroyWindow(dpy, glx_drawable);
-      glx_drawable = None;
+
+      if ( glx_drawable != xSubWindow->xwindowid )
+        {
+          glXDestroyWindow(dpy, glx_drawable);
+          glx_drawable = None;
+        }
+
       DESTROY(xSubWindow);
     }
 }
@@ -287,17 +305,22 @@ static XGGLContext *currentGLContext;
   MAKE_DISPLAY(dpy);
 
   if (context == nil || ![context isKindOfClass: [XGGLContext class]])
-    [NSException raise: NSInvalidArgumentException
-		 format: @"%@ is an invalid context", context];
+    {
+        [NSException raise: NSInvalidArgumentException
+		            format: @"%@ is an invalid context", context];
+    }
 
   glXCopyContext(dpy, ((XGGLContext *)context)->glx_context, 
                  glx_context, mask);
 
   error = glGetError();
-  if ( error != GL_NO_ERROR )
-      NSDebugMLLog( @"GLX", @"Can not copy GL context %@ from context %@ - Errror %u",
-                            self, context, error );
 
+  if ( error != GL_NO_ERROR )
+    {
+      NSDebugMLLog( @"GLX", 
+                    @"Can not copy GL context %@ from context %@ - Error %u",
+                    self, context, error );
+    }
 }
 
 - (void)createTexture:(unsigned long)target 
@@ -320,66 +343,8 @@ static XGGLContext *currentGLContext;
   MAKE_DISPLAY(dpy);
 
   glXSwapBuffers(dpy, glx_drawable);
-#if 0
-  // FIXME Not with Mesa Xlib sofware driver !
-  // TODO Use EXT_framebuffer_blit if available
-  if ( _doubleBuffer )
-    {
-      if ( _backingStore )
-        {
-          GLint rBuf,dBuf;
-          GLint viewport[4];
-          GLfloat raster_pos[4];
-
-          /* save viewport */
-          glGetIntegerv(GL_VIEWPORT, viewport);
-          /* save old raster position */
-          glGetFloatv(GL_CURRENT_RASTER_POSITION, raster_pos);
-          /* set raster position */
-          glRasterPos4f(0.0, 0.0, 0.0, 1.0);
-          /* save target/source buffers */
-          glGetIntegerv( GL_READ_BUFFER, &rBuf );
-          glGetIntegerv( GL_DRAW_BUFFER, &wBuf );
-
-          /* set projection matrix */
-          glMatrixMode(GL_PROJECTION);
-          glPushMatrix();
-          glLoadIdentity() ;
-          gluOrtho2D(0, viewport[2], 0, viewport[3]);
-          /* set modelview matrix */
-          glMatrixMode(GL_MODELVIEW);
-          glPushMatrix();
-          glLoadIdentity();
-
-          //FIXME copy depth, stencil?
-          glReadBuffer (GL_FRONT_LEFT);
-          glDrawBuffer (GL_BACK_LEFT);
-          glCopyPixels(0, 0, viewport[2], viewport[3], GL_COLOR);
-          if ( _stereo )
-            {
-              glReadBuffer (GL_FRONT_RIGHT);
-              glDrawBuffer (GL_BACK_RIGHT);
-              glCopyPixels(0, 0, viewport[2], viewport[3], GL_COLOR);
-            }
-
-          /* restore target/source buffers */
-          glReadBuffer (rBuf);
-          glDrawBuffer (dBuf);
-          /* restore old raster position */
-          glRasterPos4fv(raster_pos);
-          /* restore old matrices */
-          glPopMatrix();
-          glMatrixMode(GL_PROJECTION);
-          glPopMatrix();
-          glMatrixMode(GL_MODELVIEW);
-          /* restore viewport */
-          glViewport(viewport[0],viewport[1],viewport[2],viewport[3]);
-        }
-    }
-#endif
 }
 
-
 - (void)getValues:(long *)vals 
      forParameter:(NSOpenGLContextParameter)param
 {
@@ -387,19 +352,21 @@ static XGGLContext *currentGLContext;
   [self notImplemented: _cmd];
 }
 
-
 - (id)initWithFormat: (NSOpenGLPixelFormat *)_format 
 	    shareContext: (NSOpenGLContext *)share
 {
   self = [super init];
+
   if (!self)
-    return nil;
+    {
+      return nil;
+    }
 
   glx_context = None;
   
   if (!_format || ![_format isKindOfClass: [XGGLPixelFormat class]])
     {
-      NSDebugMLLog(@"GLX", @"invalid format %@", _format);
+      NSDebugMLLog(@"GLX", @"Invalid format %@", _format);
       RELEASE(self);
 
       return nil;
@@ -407,16 +374,15 @@ static XGGLContext *currentGLContext;
 
   ASSIGN(pixelFormat, (XGGLPixelFormat *)_format);
   
-  //FIXME: allow index mode and sharing
   glx_context = [pixelFormat createGLXContext: (XGGLContext *)share];
   
   return self;
 }
 
-
 - (void) dealloc
 {
-  NSDebugMLLog(@"GLX", @"deallocating");
+  NSDebugMLLog(@"GLX", @"Deallocating");
+
   [self _detach];
   RELEASE(pixelFormat);
 
@@ -434,27 +400,30 @@ static XGGLContext *currentGLContext;
   MAKE_DISPLAY(dpy);
 
   if (xSubWindow == nil)
-    [NSException raise: NSGenericException
-		 format: @"GL Context is not bind, cannot be made current"];
+    {
+      [NSException raise: NSGenericException
+		          format: @"GL Context has no view attached, cannot be\
+                            made current"];
+    }
   
   NSAssert(glx_context != None && glx_drawable != None,
 	   NSInternalInconsistencyException);
 
-  if (GSglxMinorVersion(dpy) >= 3)
+  if ([XGGLPixelFormat glxMinorVersion] >= 3)
     {
-      //NSDebugMLLog(@"GLX", @"before glXMakeContextCurrent");
       if ( !glXMakeContextCurrent(dpy, glx_drawable, glx_drawable, glx_context) )
-        NSDebugMLLog( @"GLX", @"Can not make GL context %@ current - Errror %u",
-                              self, glGetError() );
-      //NSDebugMLLog(@"GLX", @"after glXMakeContextCurrent");
+        {
+          NSDebugMLLog( @"GLX", @"Cannot make GL context %@ current - Error %u",
+                        self, glGetError() );
+        }
     }
   else
     {
-      //NSDebugMLLog(@"GLX", @"before glXMakeCurrent");
       if ( !glXMakeCurrent(dpy, glx_drawable, glx_context) )
-        NSDebugMLLog( @"GLX", @"Can not make GL context %@ current - Errror %u",
-                              self, glGetError() );
-      //NSDebugMLLog(@"GLX", @"after glXMakeCurrent");
+        {
+          NSDebugMLLog( @"GLX", @"Cannot make GL context %@ current - Error %u",
+                        self, glGetError() );
+        }
     }
 
   currentGLContext = self;
@@ -487,18 +456,24 @@ static XGGLContext *currentGLContext;
 - (void)setView:(NSView *)view
 {
   NSView *current_view;
+
   if (!view)
-    [NSException raise: NSInvalidArgumentException
-		 format: @"setView called with a nil value"];
+    {
+      [NSException raise: NSInvalidArgumentException
+		          format: @"setView called with a nil value"];
+    }
 
   NSAssert(pixelFormat, NSInternalInconsistencyException);
   current_view = [self view];
+
   if ( current_view != nil )
     {
       [current_view _setIgnoresBacking: saved_ignores_backing];
     }
+
   ASSIGN(xSubWindow, [XGXSubWindow subwindowOnView: view 
-                                   visualinfo: [pixelFormat xvinfo]]);
+                                        visualinfo: [pixelFormat visualinfo]]);
+
   glx_drawable = [pixelFormat drawableForWindow: xSubWindow->xwindowid];
   saved_ignores_backing = [view _ignoresBacking];
   [view _setIgnoresBacking: YES];
