@@ -22,6 +22,8 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include <config.h>
+
 #include <Foundation/NSUserDefaults.h>
 
 #include "x11/XGServer.h"
@@ -43,9 +45,11 @@ static int num_window_buffers;
 
 static int use_shape_hack = 0; /* this is an ugly hack : ) */
 
+#ifdef XSHM
 
 static int did_test_xshm = 0;
 static int use_xshm = 1;
+static Bool use_xshm_pixmaps = 0;
 static int num_xshm_test_errors = 0;
 
 static NSString *xshm_warning
@@ -73,21 +77,13 @@ static void test_xshm(Display *display, Visual *visual, int drawing_depth)
 
   /* This seems to return success if the X server understands the XShm
   protocol, regardless of whether XShm can actually be used or not... */
-  if (!XShmQueryExtension(display))
-    {
-      NSLog(@"XShm not supported by X server.");
-      NSLog(xshm_warning);
-      use_xshm = 0;
-      return;
-    }
-
   {
   int major, minor;
-  Bool pixmaps;
 
-  if (!XShmQueryVersion(display, &major, &minor, &pixmaps) || !pixmaps)
+  if (!XShmQueryExtension(display) || 
+      !XShmQueryVersion(display, &major, &minor, &use_xshm_pixmaps))
     {
-      NSLog(@"XShm pixmaps not supported by X server.");
+      NSLog(@"XShm not supported by X server.");
       NSLog(xshm_warning);
       use_xshm = 0;
       return;
@@ -174,7 +170,7 @@ no_xshm:
     XSetErrorHandler(old_error_handler);
   }
 }
-
+#endif
 
 @implementation XWindowBuffer
 
@@ -296,6 +292,7 @@ no_xshm:
       if (wi->window->xframe.size.width * wi->window->xframe.size.height < 4096)
         goto no_xshm;
 
+#ifdef XSHM
       if (!did_test_xshm)
         test_xshm(wi->display, visual, drawing_depth);
 
@@ -347,21 +344,24 @@ no_xshm:
           goto no_xshm;
         }
 
-      /* We try to create a shared pixmap using the same buffer, and set
-         it as the background of the window. This allows X to handle expose
-         events all by itself, which avoids white flashing when things are
-         dragged across a window. */
-      /* TODO: we still get and handle expose events, although we don't
-         need to. */
-      wi->pixmap = XShmCreatePixmap(wi->display, wi->drawable,
-                                    wi->ximage->data, &wi->shminfo,
-                                    wi->window->xframe.size.width,
-                                    wi->window->xframe.size.height,
-                                    drawing_depth);
-      if (wi->pixmap) /* TODO: this doesn't work */
+      if (use_xshm_pixmaps)
         {
-          XSetWindowBackgroundPixmap(wi->display, wi->window->ident,
-                                     wi->pixmap);
+          /* We try to create a shared pixmap using the same buffer, and set
+             it as the background of the window. This allows X to handle expose
+             events all by itself, which avoids white flashing when things are
+             dragged across a window. */
+          /* TODO: we still get and handle expose events, although we don't
+             need to. */
+          wi->pixmap = XShmCreatePixmap(wi->display, wi->drawable,
+                                        wi->ximage->data, &wi->shminfo,
+                                        wi->window->xframe.size.width,
+                                        wi->window->xframe.size.height,
+                                        drawing_depth);
+          if (wi->pixmap) /* TODO: this doesn't work */
+            {
+              XSetWindowBackgroundPixmap(wi->display, wi->window->ident,
+                                         wi->pixmap);
+            }
         }
 
       /* On some systems (eg. freebsd), X can't attach to the shared segment
@@ -373,6 +373,7 @@ no_xshm:
       actually be destroyed, but if we crashed before doing this, it wouldn't
       be destroyed despite nobody being attached anymore. */
       shmctl(wi->shminfo.shmid, IPC_RMID, 0);
+#endif
 
       if (!wi->ximage)
         {
@@ -384,6 +385,14 @@ no_xshm:
                                     wi->window->xframe.size.height,
                                     8, 0);
 
+	  /* Normally, the data of an XImage is saved with the X server's
+	     byte order. However, some backends (notably cairo) use the
+	     native byte order of the client for the backend image, hence
+	     we must set the image's byte order here. Without this, GNUstep
+	     applications would display wrong colors on an X server that
+	     is running on a machine with a different byte order than the
+	     client (bug #28590). */
+	  wi->ximage->byte_order = wi->DI.byte_order;
           wi->ximage->data = malloc(wi->ximage->height * wi->ximage->bytes_per_line);
           if (!wi->ximage->data)
             {
