@@ -229,6 +229,8 @@ static NSArray *faFromFc(FcPattern *pat)
     return @"Bitstream Vera Sans";
   if ([allFontNames containsObject: @"FreeSans"])
     return @"FreeSans";
+  if ([allFontNames containsObject: @"DejaVu Sans"])
+    return @"DejaVu Sans";
   return @"Helvetica";
 }
 
@@ -238,6 +240,8 @@ static NSArray *faFromFc(FcPattern *pat)
     return @"Bitstream Vera Sans-Bold";
   if ([allFontNames containsObject: @"FreeSans-Bold"])
     return @"FreeSans-Bold";
+  if ([allFontNames containsObject: @"DejaVu Sans-Bold"])
+    return @"DejaVu Sans-Bold";
   return @"Helvetica-Bold";
 }
 
@@ -247,6 +251,8 @@ static NSArray *faFromFc(FcPattern *pat)
     return @"Bitstream Vera Sans Mono";
   if ([allFontNames containsObject: @"FreeMono"])
     return @"FreeMono";
+  if ([allFontNames containsObject: @"DejaVu Sans Mono"])
+    return @"DejaVu Sans Mono";
   return @"Courier";
 }
 
@@ -263,7 +269,7 @@ static NSArray *faFromFc(FcPattern *pat)
 @implementation GSXftFontInfo
 
 - initWithFontName: (NSString*)name
-	    matrix: (const float *)fmatrix
+	    matrix: (const CGFloat*)fmatrix
 	screenFont: (BOOL)screenFont
 {
   if (screenFont)
@@ -368,7 +374,7 @@ Ones(unsigned int n)
   return coveredCharacterSet;
 }
 
-- (float) widthOfString: (NSString*)string
+- (CGFloat) widthOfString: (NSString*)string
 {
   XGlyphInfo extents;
   int len = [string length];
@@ -384,7 +390,7 @@ Ones(unsigned int n)
   return extents.width;
 }
 
-- (float) widthOfGlyphs: (const NSGlyph *) glyphs length: (int) len
+- (CGFloat) widthOfGlyphs: (const NSGlyph *) glyphs length: (int) len
 {
   XGlyphInfo extents;
   XftChar16 buf[len];
@@ -471,7 +477,7 @@ Ones(unsigned int n)
 }
 
 /*
-- (float) pointSize
+- (CGFloat) pointSize
 {
   Display	*xdpy = [XGServer currentXDisplay];
 
@@ -540,7 +546,7 @@ Ones(unsigned int n)
     }
 }
 
-- (float) widthOf: (const char*) s length: (int) len
+- (CGFloat) widthOf: (const char*) s length: (int) len
 {
   XGlyphInfo extents;
 
