@@ -15,18 +15,20 @@
    This file is part of the GNUstep GUI X/GPS Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #include <config.h>
@@ -74,12 +76,11 @@ cache_name()
       NSFileManager *mgr;
       BOOL flag;
       Display *dpy = [XGServer currentXDisplay];
-      char *display_name = DisplayString(dpy);
       NSString *file_name;
       NSArray *paths;
       NSString *path = nil;
 
-      file_name = [NSString stringWithCString: XDisplayName(display_name)];
+      file_name = XGFontCacheName(dpy);
 
       paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,
                                                   NSUserDomainMask, YES);
@@ -149,7 +150,7 @@ load_cache(NSString *cacheName, BOOL async)
 
   if ((o == nil)
     || ((v = [o objectForKey: @"Version"]) == nil)
-    || ([v intValue] != 2))
+    || ([v intValue] != 3))
     {
       NSString *file_name = [cacheName lastPathComponent];
       NSString *path;
