@@ -1,6 +1,6 @@
 /* XGGState - Implements graphic state drawing for Xlib
 
-   Copyright (C) 1998 Free Software Foundation, Inc.
+   Copyright (C) 1998-2010 Free Software Foundation, Inc.
 
    Written by:  Adam Fedor <fedor@gnu.org>
    Date: Nov 1998
@@ -24,17 +24,22 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include <Foundation/NSObjCRuntime.h>
-#include <AppKit/NSBezierPath.h>
-#include <AppKit/NSFont.h>
-#include <AppKit/NSGraphics.h>
+#import "config.h"
+#import <Foundation/NSObjCRuntime.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSValue.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <AppKit/NSBezierPath.h>
+#import <AppKit/NSFont.h>
+#import <AppKit/NSGraphics.h>
 
-#include "xlib/XGGeometry.h"
-#include "xlib/XGContext.h"
-#include "xlib/XGGState.h"
-#include "xlib/XGContext.h"
-#include "xlib/XGPrivate.h"
+#import "xlib/XGGeometry.h"
+#import "xlib/XGContext.h"
+#import "xlib/XGGState.h"
+#import "xlib/XGContext.h"
+#import "xlib/XGPrivate.h"
 #include "math.h"
 
 #define XDPY (((RContext *)context)->dpy)
@@ -771,30 +776,6 @@ static Region emptyRegion;
     }
 }
 
-- (void) compositeGState: (GSGState *)source 
-                fromRect: (NSRect)aRect
-                 toPoint: (NSPoint)aPoint
-                      op: (NSCompositingOperation)op
-{
-  [self compositeGState: source 
-               fromRect: aRect
-                toPoint: aPoint
-                     op: op
-               fraction: 1.0];
-}
-
-- (void) dissolveGState: (GSGState *)source
-               fromRect: (NSRect)aRect
-                toPoint: (NSPoint)aPoint 
-                  delta: (float)delta
-{
-  [self compositeGState: source 
-               fromRect: aRect
-                toPoint: aPoint
-                     op: NSCompositeSourceOver
-               fraction: delta];
-}
-
 - (void) compositerect: (NSRect)aRect
                     op: (NSCompositingOperation)op
 {
@@ -1486,6 +1467,12 @@ static Region emptyRegion;
 
 - (void)DPSeofill 
 {
+  if (pattern != nil)
+    {
+      [self eofillPath: path withPattern: pattern];
+      return;
+    }
+
   if ((cstate & COLOR_FILL) == 0)
     [self setColor: &fillColor state: COLOR_FILL];
 
@@ -1494,6 +1481,12 @@ static Region emptyRegion;
 
 - (void)DPSfill 
 {
+  if (pattern != nil)
+    {
+      [self fillPath: path withPattern: pattern];
+      return;
+    }
+
   if ((cstate & COLOR_FILL) == 0)
     [self setColor: &fillColor state: COLOR_FILL];
 
@@ -1546,6 +1539,12 @@ static Region emptyRegion;
       return;
     }
 
+  if (pattern != nil)
+    {
+      [self fillRect: NSMakeRect(x, y, w, h) withPattern: pattern];
+      return;
+    }
+
   if ((cstate & COLOR_FILL) == 0)
     [self setColor: &fillColor state: COLOR_FILL];
 
@@ -1903,3 +1902,29 @@ NSDebugLLog(@"XGGraphics", @"Fill %@ X rect %d,%d,%d,%d",
 }
 
 @end
+
+@implementation XGGState (PatternColor)
+
+- (void *) saveClip
+{
+  if (clipregion)
+    {
+      Region region = XCreateRegion();
+
+      XIntersectRegion(clipregion, clipregion, region);
+      return region;
+    }
+  return clipregion;
+}
+
+- (void) restoreClip: (void *)savedClip
+{
+  if (clipregion)
+    {
+      XDestroyRegion(clipregion);
+    }
+  clipregion = savedClip;
+  [self setClipMask];
+}
+
+@end
